/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.AjaxCommon;
import com.sun.jmaki.AjaxContext;
import com.sun.jmaki.AjaxWrapperPhaseListener;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import javax.faces.component.ActionSource;
import javax.faces.component.EditableValueHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.ValueHolder;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * <p>Abstract base class for renderer implementations.</p>
 *
 * <p><strong>IMPLEMENTATION NOTES</strong> - This class supports
 * many convenience helper methods useful to <code>Renderer</code>
 * implementors, as well as default implementations of the key
 * <code>Renderer</code> processing methods:</p>
 * <ul>
 * <li><code>decode()</code> - Update the submitted value if the
 *     component is an instance of <code>EditableValueHolder</code>
 *     and the component is not disabled or set to readonly.</li>
 * <li><code>encodeBegin()</code> - If this is an asynchronous
 *     callback request for values, do nothing.  Otherwise, call
 *     <code>outerBegin()</code> followed by either
 *     <code>designBegin()</code> or <code>runtimeBegin()</code>
 *     based on whether we are inside a tool or operating at runtime.</li>
 * <li><code>encodeChildren()</code> - If this is an asynchronous
 *     callback request for values, do nothing.  Otherwise, call
 *     <code>designChildren()</code> or <code>runtimeChildren()</code>
 *     based on whether we are inside a tool or operating at runtime.</li>
 * <li><code>encodeEnd()</code> - If this is an asynchronous
 *     callback request for values, render the value and return.
 *     Otherwise, call either <code>designEnd()</code> or
 *     <code>runtimeEnd()</code> based on whether we are inside a
 *     tool or operating at runtime, then call <code>outerEnd()</code>.</li>
 * </ul>
 *
 * <p>The <code>outerBegin()</code> and <code>outerEnd()</code>
 * methods generate a <code>&lt;div&gt;</code> element with pass
 * throughs for the <code>id</code>, <code>style</code>, and
 * <code>styleClass</code> properties as required for well behaved
 * visual components.  These methods are called at the beginning
 * and end of rendering, whether or not we are inside a tool.</p>
 *
 * <p>The <code>designBegin()</code>, <code>designChildren()</code>,
 * and <code>designEnd()</code> methods are called from the
 * correspondingly named <code>encodeXxx()</code> methods, when we
 * are inside a tool.  The default implementations delegate to the
 * corresponding <code>runtimeXxx()</code> methods, so if your design
 * time and runtime rendering requirements are identical, there is
 * no need to override these methods.</p>
 *
 * <p>The <code>runtimeBegin()</code>, <code>runtimeChildren()</code>,
 * and <code>runtimeEnd()</code> methods are called from the
 * correspondingly named <code>encodeXxx()</code> methods, when we
 * are executing at runtime.  The default implementations of
 * <code>runtimeBegin()</code> and <code>runtimeChildren()</code>
 * do nothing, while the default implementation of <code>runtimeEnd()</code>:</p>
 * <ul>
 * <li>Calls <code>paramValues()</code> to accumulate parameters to
 *     be passed to the jMaki rendering machinery.  This will in
 *     turn call other <code>paramXxx()</code> methods that may
 *     optionally be overridden to customize the values to be
 *     transmitted.</li>
 * <li>Uses this value to construct a <code>AjaxContext</code>.</li>
 * <li>Passes the <code>AjaxContext</code> to the <code>doProcess()</code>
 *     method of <code>AjaxCommon</code>.</li>
 * </ul>
 *
 * <p>To customize the runtime rendering of a component, you must
 * generally override <code>runtimeEnd()</code>, and also possibly
 * provide implementations for <code>runtimeBegin()</code> and/or
 * <code>runtimeChildren()</code>.</p>
 */
public abstract class AbstractRenderer extends Renderer {


    // ------------------------------------------------------ Manifest Constants


    /**
     * <p>The names of boolean value attributes that should simply
     * be passed through.</p>
     */
    protected static final String[] BOOLEAN_ATTRIBUTES =
    { "disabled", "readonly", };


    /**
     * <p>The names of integer value attributes that should simply
     * be passed through.</p>
     */
    protected static final String[] INTEGER_ATTRIBUTES =
    { };


    /**
     * <p>The names of string value attributes that should simply
     * be passed through.</p>
     */
    protected static final String[] STRING_ATTRIBUTES =
    { "style", };


    /**
     * <p>Generic attribute on the view root where we maintain a
     * <code>Set</code> of widget names for which a link to the
     * component-specific stylesheet has already been rendered.</p>
     */
    protected static final String WIDGET_NAMES =
            "com.sun.jmaki.faces.WIDGET_NAMES";


    // -------------------------------------------------------- Renderer Methods


    /**
     * <p>Common logic to process submitted values on <code>ActionSource</code>
     * and <code>EditableValueHolder</code> components.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being decoded
     */
    public void decode(FacesContext context, UIComponent component) {

        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

        if (component instanceof ActionSource) {
            ; // FIXME - common handling for ActionSource components
        } else if (component instanceof EditableValueHolder) {
            if (!component.isRendered() || isDisabled(component) || isReadOnly(component)) {
                return;
            }
            String clientId = component.getClientId(context);
            Map map = context.getExternalContext().getRequestParameterMap();
            ((EditableValueHolder) component).setSubmittedValue(map.get(clientId));
        }

    }


    /**
     * <p>Render the beginning of the specified widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     *
     * @exception IOException if an input/output error occurs
     */
    public void encodeBegin(FacesContext context, UIComponent component)
      throws IOException {

        // Skip rendering if it is turned off
        if (!component.isRendered()) {
            return;
        }

        // If this an asynchronous callback for our value, do nothing here
        // because the value will be rendered in encodeEnd()
        if (valueRequest(context, component)) {
            return;
        }

        // Prepare the local variables we will need
        ResponseWriter writer = context.getResponseWriter();

        // Render differently at design time versus runtime
        outerBegin(context, component, writer);
        if (Beans.isDesignTime()) {
            designCSS(context, component, writer);
            designBegin(context, component, writer);
        } else {
            runtimeBegin(context, component, writer);
        }

    }


    /**
     * <p>Render the children of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     *
     * @exception IOException if an input/output error occurs
     */
    public void encodeChildren(FacesContext context, UIComponent component)
      throws IOException {

        // Skip rendering if it is turned off
        if (!component.isRendered()) {
            return;
        }

        // If this an asynchronous callback for our value, do nothing here
        // because the value will be rendered in encodeEnd()
        if (valueRequest(context, component)) {
            return;
        }

        // Prepare the local variables we will need
        ResponseWriter writer = context.getResponseWriter();

        // Render the children of this widget
        if (Beans.isDesignTime()) {
            designChildren(context, component, writer);
        } else {
            runtimeChildren(context, component, writer);
        }


    }


    /**
     * <p>Render the end of the specified widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     *
     * @exception IOException if an input/output error occurs
     */
    public void encodeEnd(FacesContext context, UIComponent component)
      throws IOException {

        // Skip rendering if it is turned off
        if (!component.isRendered()) {
            return;
        }

        // Prepare the local variables we will need
        ResponseWriter writer = context.getResponseWriter();

        // If this an asynchronous callback for our value, render just that
        if (valueRequest(context, component)) {
            writer.write(valueString(context, component));
            return;
        }

        // Render differently at design time versus runtime
        if (Beans.isDesignTime()) {
            designEnd(context, component, writer);
        } else {
            runtimeEnd(context, component, writer);
        }
        outerEnd(context, component, writer);

    }


    // ------------------------------------------------------- Protected Methods


    /**
     * <p>Render the <em>begin</em> design time representation of this
     * component.  The default implementation delegates to the
     * <code>runtimeBegin()</code> method, so you need not override this
     * method if design time and runtime rendering is the same.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designBegin(FacesContext context, UIComponent component,
                               ResponseWriter writer) throws IOException {

        runtimeBegin(context, component, writer);

    }


    /**
     * <p>Render the <em>children</em> design time representation of this
     * component.  The default implementation delegates to the
     * <code>runtimeChildren()</code> method, so you need not override this
     * method if design time and runtime rendering is the same.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designChildren(FacesContext context, UIComponent component,
                                  ResponseWriter writer) throws IOException {

        runtimeChildren(context, component, writer);

    }


    /**
     * <p>Render a link to the component-specific stylesheet for the
     * widget represented by the specified component.  Such a link will
     * be rendered at most once per page.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designCSS(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        // Render a link to the component stylesheet (once per page maximum)
        String name = paramName(context, component).replace('.', '/');
/* FIXME - no way to clear the set at the end of "rendering" the page???
        Map map = context.getViewRoot().getAttributes();
        Set set = (Set) map.get(WIDGET_NAMES);
        if (set == null) {
            set = new HashSet();
            map.put(WIDGET_NAMES, set);
        }
        if (!set.contains(name)) {
*/
            // FIXME - adjust this if non-default "resources" directory used
            URL url = context.getExternalContext().getResource
              ("/resources/" + name + "/component.css");
            // FIXME - adjust this if nesting in jmaki-resources.jar changes!
            if (url == null) {
                url = this.getClass().getResource
                  ("/META-INF/resources/" + name + "/component.css");
            }
            if (url != null) {
                writer.startElement("link", component);
                writer.writeAttribute("rel", "stylesheet", null);
                writer.writeAttribute("type", "text/css", null);
                writer.writeURIAttribute("href", url.toExternalForm(), null);
                writer.endElement("link");
                writer.write("\n");
            }
/*
            set.add(name);
        }
*/

    }


    /**
     * <p>Render the <em>end</em> design time representation of this
     * component.  The default implementation delegates to the
     * <code>runtimeEnd()</code> method, so you need not override this
     * method if design time and runtime rendering is the same.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        runtimeEnd(context, component, writer);

    }


    /**
     * <p>Render the specified component and its children recursively.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> to be rendered
     *
     * @exception IOException if an input/output error occurs
     */
    protected void encodeRecursive(FacesContext context, UIComponent component)
      throws IOException {

        component.encodeBegin(context);
        if (component.getRendersChildren()) {
            component.encodeChildren(context);
        } else {
            Iterator kids = component.getChildren().iterator();
            while (kids.hasNext()) {
                UIComponent kid = (UIComponent) kids.next();
                encodeRecursive(context, kid);
            }
        }
        component.encodeEnd(context);

    }



    /**
     * <p>Return <code>true</code> if a boolean <code>disabled</code>
     * propety is set to <code>true</code> for the specified component.</p>
     *
     * @param component <code>UIComponent</code> to be checked
     */
    protected boolean isDisabled(UIComponent component) {

        Object value = component.getAttributes().get("disabled");
        if ((value != null) && (value instanceof Boolean)) {
            return ((Boolean) value).booleanValue();
        } else {
            return false;
        }

    }


    /**
     * <p>Return <code>true</code> if a boolean <code>readonly</code>
     * propety is set to <code>true</code> for the specified component.</p>
     *
     * @param component <code>UIComponent</code> to be checked
     */
    protected boolean isReadOnly(UIComponent component) {

        Object value = component.getAttributes().get("readonly");
        if ((value != null) && (value instanceof Boolean)) {
            return ((Boolean) value).booleanValue();
        } else {
            return false;
        }

    }


    /**
     * <p>Render the outer beginning of this component, independent of whether
     * we are in design time or not.  The default implementation opens a
     * <code>&lt;div&gt;</code> element and passes through the client id,
     * as well as the <code>style</code> and <code>styleClass</code> attributes
     * if present.</p>
     *
     * @param context <code>FacesContext for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> we are writing to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void outerBegin(FacesContext context, UIComponent component,
                              ResponseWriter writer) throws IOException {

        writer.startElement("div", component);
        writer.writeAttribute("id", component.getClientId(context) + "_widget", "id");
        passThroughString(component, STRING_ATTRIBUTES, writer);
        String value = (String) component.getAttributes().get("styleClass");
        if (value != null) {
            writer.writeAttribute("class", value, "styleClass");
        }
        writer.write("\n");

    }


    /**
     * <p>Render the outer ending of this component, independent of whether
     * we are in design time or not.  The default implementation closes
     * the <code>&lt;div&gt;</code> element opened in the
     * <code>outerBegin()</code> method.</p>
     *
     * @param context <code>FacesContext for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> we are writing to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void outerEnd(FacesContext context, UIComponent component,
                            ResponseWriter writer) throws IOException {

        writer.endElement("div");

    }


    /**
     * <p>Return the value to be passed for the <code>args</code>
     * parameter, if any.  The default implementation passes the corresponding
     * component property (if any) as a String.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        return (String) component.getAttributes().get("args");

    }


    /**
     * <p>Return the value to be passed for the <code>name</code>
     * parameter, if any.  The default implementation passes the corresponding
     * component property (if any) as a String.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return (String) component.getAttributes().get("name");

    }


    /**
     * <p>Return the value to be passed for the <code>selected</code>
     * parameter, if any.  The default implementation passes the corresponding
     * component property (if any) as a String.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramSelected(FacesContext context, UIComponent component) {

        return (String) component.getAttributes().get("selected");

    }


    /**
     * <p>Return the value to be passed for the <code>template</code>
     * parameter, if any.  The default implementation passes the corresponding
     * component property (if any) as a String.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramTemplate(FacesContext context, UIComponent component) {

        return (String) component.getAttributes().get("template");

    }


    /**
     * <p>Return the value to be passed for the <code>uuid</code>
     * parameter, if any.  The default implementation passes the client id
     * component property as a String.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramUuid(FacesContext context, UIComponent component) {

        return component.getClientId(context);

    }


    /**
     * <p>Return the value to be passed for the <code>value</code>
     * parameter, if any.  The default implementation passes the
     * value returned by <code>valueString()</code>, if any.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramValue(FacesContext context, UIComponent component) {

        return valueString(context, component);

    }


    /**
     * <p>Return a <code>Map</code> of parameter values that will be
     * passed to the jMaki rendering machinery for configuring the
     * widget represented by this component.  The default implementation
     * calculates the following path related values:</p>
     * <ul>
     * <li><code>jsfRoot</code> - FIXME -- definition</li>
     * <li><code>resourceBase</code> - FIXME -- definition</li>
     * <li><code>service</code> - FIXME -- definition</li>
     * <li><code>webRoot</code> - FIXME -- definition</li>
     * </ul>
     *
     * <p>Then, it calls through to set the values of the following
     * values:</p>
     * <ul>
     * <li><code>args</code> -- value returned by <code>paramArgs()</code></li>
     * <li><code>name</code> -- value returned by <code>paramName()</code></li>
     * <li><code>selected</code> -- value returned by <code>paramSelected()</code></li>
     * <li><code>template</code> -- value returned by <code>paramTemplate()</code></li>
     * <li><code>uuid</code> -- value returned by <code>paramUuid()</code></li>
     * <li><code>value</code> -- value returned by <code>paramValue()</code></li>
     * </ul>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected Map paramValues(FacesContext context, UIComponent component) {

        // Initialize some values we will need
        Map map = new HashMap();
        map.put("jsfRoot", "/faces"); // FIXME - only works for prefix mapping!

        // Fill in values acquired from other (possibly overridden) methods
        map.put("args", paramArgs(context, component));
        map.put("name", paramName(context, component));
        map.put("selected", paramSelected(context, component));
        map.put("template", paramTemplate(context, component));
        map.put("uuid", paramUuid(context, component));
        map.put("value", paramValue(context, component));

        // Return the resulting map
        return map;

    }


    /**
     * <p>Pass through the values of the specified boolean attributes,
     * if they exist on the specified component.</p>
     *
     * @param component <code>UIComponent</code> being rendered
     * @param names Names of the relevant attributes
     * @param writer <code>ResponseWriter</code> to use for writing
     *
     * @exception IOException if an input/output error occurs
     */
    protected void passThroughBoolean(UIComponent component, String[] names,
                                      ResponseWriter writer) throws IOException {

        boolean result = false;
        Object value = null;
        for (int i = 0; i < names.length; i++) {
            value = component.getAttributes().get(names[i]);
            if (value != null) {
                if (value instanceof Boolean) {
                    result = ((Boolean) value).booleanValue();
                }
                if (result) {
                    writer.writeAttribute(names[i], names[i], names[i]);
                }
            }
        }

    }


    /**
     * <p>Pass through the values of the specified integer attributes,
     * if they exist on the specified component.</p>
     *
     * @param component <code>UIComponent</code> being rendered
     * @param names Names of the relevant attributes
     * @param writer <code>ResponseWriter</code> to use for writing
     *
     * @exception IOException if an input/output error occurs
     */
    protected void passThroughInteger(UIComponent component, String[] names,
                                      ResponseWriter writer) throws IOException {

        Object value = null;
        for (int i = 0; i < names.length; i++) {
            value = component.getAttributes().get(names[i]);
            if (value != null) {
                if (value instanceof Integer) {
                    writer.writeAttribute
                      (names[i], "" + ((Integer) value).intValue(), names[i]);
                }
            }
        }

    }


    /**
     * <p>Pass through the values of the specified string attributes,
     * if they exist on the specified component.</p>
     *
     * @param component <code>UIComponent</code> being rendered
     * @param names Names of the relevant attributes
     * @param writer <code>ResponseWriter</code> to use for writing
     *
     * @exception IOException if an input/output error occurs
     */
    protected void passThroughString(UIComponent component, String[] names,
                                     ResponseWriter writer) throws IOException {

        Object value = null;
        for (int i = 0; i < names.length; i++) {
            value = component.getAttributes().get(names[i]);
            if (value != null) {
                if (value instanceof String) {
                    writer.writeAttribute(names[i], value, names[i]);
                }
            }
        }

    }


    /**
     * <p>Return the specified string surrounded with either single
     * or double quotes, with any embedded quotes suitably escaped.</p>
     *
     * @param value The string to be quoted and returned
     */
    protected String quotedString(String value) {

        if (value.indexOf("\"") < 0) {
            return '"' + value + '"';
        } else if (value.indexOf("'") < 0) {
            return "'" + value + "'";
        }
        StringBuffer sb = new StringBuffer("\"");
        for (int i = 0; i < value.length(); i++) {
            char ch = value.charAt(i);
            if (ch == '"') {
                sb.append('\\');
            }
            sb.append(ch);
        }
        sb.append("\"");
        return sb.toString();

    }


    /**
     * <p>Render the <em>begin</em> runtime representation of this
     * component.  The default implementation renders nothing.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeBegin(FacesContext context, UIComponent component,
                                ResponseWriter writer) throws IOException {

        ;

    }


    /**
     * <p>Render the <em>children</em> runtime representation of this
     * component.  The default implementation renders nothing.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeChildren(FacesContext context, UIComponent component,
                                   ResponseWriter writer) throws IOException {

        ;

    }


    /**
     * <p>Render the <em>end</em> runtime representation of this
     * component.  The default implementation accumulates parameter
     * values by calling <code>paramValues()</code>, uses that to
     * construct an <code>AjaxContext</code>, and then passes this
     * context to <code>AjaxCommon.doProcess()</code>.</p>
     *
     * <p><strong>IMPLEMENTATION NOTE</strong> - some jMaki widgets
     * support the use of an external HTML template file to specify
     * their content.  To better fit the JSF paradigm, you can use
     * the output rendered by the children of a component as follows:</p>
     * <ul>
     * <li>In the <code>runtimeBegin()</code> method of the component
     *     being rendered, replace the <code>ResponseWriter</code> for
     *     this request with an instance of <code>InterceptingResponseWriter</code>
     *     that wraps it.</li>
     * <li>From that point on, rendered output from this component, and
     *     all of its children, is accumulated into a buffer.</li>
     * <li>The code here in <code>runtimeEnd</code> will detect that the
     *     current <code>ResponseWriter</code> is an instance of our
     *     interceptor.  It will replace the interceptor with the original
     *     <code>ResponseWriter</code> instance, and pass the buffered
     *     content as the <code>template</code> property of the
     *     <code>AjaxContext</code> that is constructed.  In turn,
     *     this will cause the content to be emitted by jMaki inline,
     *     without requiring a reference to a separate URL.</li>
     * </ul>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent being rendered
     * @param writer <code>ResponseWriter to use for rendering
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeEnd(FacesContext context, UIComponent component,
                              ResponseWriter writer) throws IOException {

        // Accumulate the values we will be passing in to jMaki
        Map map = paramValues(context, component);
        StringBuffer template = null;
        if (writer instanceof InterceptingResponseWriter) {
            template = ((InterceptingResponseWriter) writer).toStringBuffer();
            writer = ((InterceptingResponseWriter) writer).getWrappedResponseWriter();
            context.setResponseWriter(writer);
        }

        // Construct an AjaxContext and tell jMaki to emit this widget
        ExternalContext econtext = context.getExternalContext();
        AjaxContext ctx =
          new AjaxContext((ServletContext) econtext.getContext(),
                          (HttpServletRequest) econtext.getRequest(),
                          (HttpServletResponse) econtext.getResponse(),
                          writer, map);
        if (template != null) {
            ctx.setTemplate(template);
        }
        AjaxCommon.doProcess(ctx);

    }


    /**
     * <p>Return <code>true</code> if this is an asynchronous callback that is
     * requesting our current value.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected boolean valueRequest(FacesContext context, UIComponent component) {

        String header = (String)
          context.getExternalContext().getRequestHeaderMap().get("com.sun.faces.avatar.partial");
        return (header != null) && header.equalsIgnoreCase("values");

    }


    /**
     * <p>Return the <code>value</code> property of a <code>ValueHolder</code>
     * component, converted to a string.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> component whose value is to
     *  be converted and returned
     */
    protected String valueString(FacesContext context, UIComponent component) {

        // Non-ValueHolder components have no value, so return empty String
        if (!(component instanceof ValueHolder)) {
            return "";
        }

        // Handle the simple cases -- null values or values that are alredy strings
        Object value = ((ValueHolder) component).getValue();
        if (value == null) {
            return "";
        }
        if (value instanceof String) {
            return (String) value;
        }

        // Look up the Converter to be used (if any)
        Converter converter = ((ValueHolder) component).getConverter();
        if (converter == null) {
            converter = context.getApplication().createConverter(value.getClass());
        }

        // Perform the appropriate conversion
        if (converter != null) {
            return converter.getAsString(context, (UIComponent) component, value);
        } else {
            return value.toString();
        }

    }


}
