/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;
import javax.faces.component.UIComponent;
import javax.faces.context.ResponseWriter;

/**
 * <p><code>ResponseWriter</code> implementation that accumulates rendered
 * output into a <code>StringBuffer</code> that can be retrieved by calling
 * <code>toStringBuffer()</code>.</p>
 */
public final class InterceptingResponseWriter extends ResponseWriter {
    

    // ------------------------------------------------------------ Constructors


    /**
     * <p>Construct a new intercepting response writer wrapping the specified
     * response writer.</p>
     *
     * @param writer Original <code>ResponseWriter</code> to be wrapped
     */
    public InterceptingResponseWriter(ResponseWriter writer) {

        this.writer = writer;

    }


    // -------------------------------------------------------- Static Variables


    /**
     * <p><code>ResourceBundle</code> for localized messages.</p>
     */
    private static final ResourceBundle bundle
            = ResourceBundle.getBundle("com.sun.jmaki.faces.renderer.Bundle");


    // ------------------------------------------------------ Instance Variables


    /**
     * <p>Flag indicating that we have begun the start of an element, but have
     * not ended it yet.</p>
     */
    private boolean begun = false;


    /**
     * <p>The <code>StringBuffer</code> into which we accumulate rendered
     * output.</p>
     */
    private StringBuffer sb = new StringBuffer();


    /**
     * <p>Original <code>ResponseWriter</code> that we are wrapping.</p>
     */
    private ResponseWriter writer = null;


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Clear the accumulated rendered output of this response writer.</p>
     */
    public void clear() {

        begun = false;
        sb.setLength(0);

    }


    /**
     * <p>Return the <code>ResponseWriter</code> instance that we
     * are wrapping.</p>
     */
    public ResponseWriter getWrappedResponseWriter() {

        return writer;

    }


    /**
     * <p>Return the accumulated contents of this response writer.</p>
     */
    public StringBuffer toStringBuffer() {

        return this.sb;

    }



    // -------------------------------------------------- ResponseWriter Methods


    /** {@inheritDoc} */
    public String getCharacterEncoding() {

        return writer.getCharacterEncoding();

    }


    /** {@inheritDoc} */
    public String getContentType() {

        return writer.getContentType();

    }


    /** {@inheritDoc} */
    public void flush() throws IOException {

        end();

    }


    /** {@inheritDoc} */
    public void startDocument() throws IOException {

        ; // Do nothing

    }


    /** {@inheritDoc} */
    public void endDocument() throws IOException {

        end();

    }


    /** {@inheritDoc} */
    public void startElement(String name, UIComponent component)
      throws IOException {

        if (name == null) {
            throw new NullPointerException(bundle.getString("element.required"));
        }
        end();
        begin();
        sb.append(name);

    }


    /** {@inheritDoc} */
    public void endElement(String name) throws IOException {

        if (name == null) {
            throw new NullPointerException(bundle.getString("element.required"));
        }
        if (begun) {
            sb.append('/');
            end();
        } else {
            sb.append("</");
            sb.append(name);
            sb.append('>');
        }

    }


    /** {@inheritDoc} */
    public void writeAttribute(String name, Object value, String property)
      throws IOException {

        if (!begun) {
            throw new NullPointerException(bundle.getString("element.missing"));
        }
        if (name == null) {
            throw new NullPointerException(bundle.getString("attribute.required"));
        }
        if (value == null) {
            return;
        }
        if (value.getClass() == Boolean.class) {
            if (Boolean.TRUE.equals(value)) {
                sb.append(' ');
                sb.append(name);
                sb.append("=\"");
                sb.append(name);
                sb.append('"');
            }
        } else {
            sb.append(' ');
            sb.append(name);
            sb.append("=");
            sb.append(filter(value));
        }

    }


    /** {@inheritDoc} */
    public void writeURIAttribute(String name, Object value, String property)
      throws IOException {

        writeAttribute(name, value, property);

    }


    /** {@inheritDoc} */
    public void writeComment(Object comment)
      throws IOException {

        end();
        sb.append("<-- ");
        if (comment != null) {
            if (comment instanceof String) {
                sb.append((String) comment);
            } else {
                sb.append(comment.toString());
            }
        }
        sb.append(" -->");

    }


    /** {@inheritDoc} */
    public void writeText(Object text, String property)
      throws IOException {

        end();
        if (text != null) {
            if (text instanceof String) {
                sb.append((String) text);
            } else {
                sb.append(text.toString());
            }
        }

    }


    /** {@inheritDoc} */
    public void writeText(Object text, UIComponent component, String property)
      throws IOException {

        writeText(text, property);

    }


    /** {@inheritDoc} */
    public void writeText(char text[], int off, int len)
      throws IOException {

        end();
        int n = off + len;
        for (int i = off; i < n; i++) {
            sb.append(text[i]);
        }

    }


    /** {@inheritDoc} */
    public ResponseWriter cloneWithWriter(Writer writer) {

        throw new UnsupportedOperationException();

    }


    // ---------------------------------------------------------- Writer Methods


    /** {@inheritDoc} */
    public void close() {

        end();

    }


    /** {@inheritDoc} */
    public void write(char[] chars, int off, int len) throws IOException {

        writeText(chars, off, len);

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Begin a new open element.</p>
     */
    private void begin() {

        sb.append('<');
        begun = true;

    }


    /**
     * <p>End a currently open element, if there is one.</p>
     */
    private void end() {

        if (begun) {
            sb.append('>');
            begun = false;
        }

    }


    /**
     * <p>Return the specified value as a quoted string, filtering
     * internal quote characters as necessary.</p>
     *
     * @param value Value to be filtered
     */
    private String filter(Object value) {

        String s = null;
        if (value instanceof String) {
            s = (String) value;
        } else {
            s = value.toString();
        }
        if (s.indexOf('"') < 0) {
            return '"' + s + '"';
        } else if (s.indexOf('\'') < 0) {
            return "'" + s + "'";
        }
        StringBuffer b = new StringBuffer(s.length() + 5);
        b.append('"');
        for (int i = 0; i < s.length(); i++) {
            char ch = s.charAt(i);
            if (ch == '\'') {
                b.append("&quot;");
            } else {
                b.append(ch);
            }
        }
        b.append('"');
        return b.toString();

    }



}
