/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * <p>Helper class to assist in creating strings in JavaScript
 * <em>object literal</em> notation.  Simply call an <code>add()</code>
 * method for each value to be included, then call <code>toString()</code>
 * to return the object literal string.</p>
 *
 * <p><strong>IMPLEMENATION NOTE</strong> - The order that elements
 * are listed in the object literal string are <strong>NOT</strong>
 * guaranteed to be in the order that the corresponding <code>add()</code>
 * methods were called.</p>
 *
 * <p><strong>IMPLEMENTATION NOTE</strong> - It is <strong>NOT</strong>
 * safe to call an <code>add()</code> method on more than one thread
 * at the same time.</p>
 */
public class ObjectLiteral {
    

    // ------------------------------------------------------ Instance Variables


    /**
     * <p><code>Map</code> containing our name/value pairs.</p>
     */
    private Map map = new HashMap();


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Add a boolean value to this object literal.</p>
     *
     * @param name Name of this field
     * @param value Value of this field
     */
    public void add(String name, boolean value) {

        map.put(name, (value == true) ? Boolean.TRUE : Boolean.FALSE);

    }


    /**
     * <p>Add an integer value to this object literal.</p>
     *
     * @param name Name of this field
     * @param value Value of this field
     */
    public void add(String name, int value) {

        map.put(name, new Integer(value));

    }


    /**
     * <p>Add a nested object literal value to this object literal,
     * unless the value is null, in which case it is ignored.</p>
     *
     * @param name Name of this field
     * @param value Value of this field
     */
    public void add(String name, ObjectLiteral value) {

        if (value != null) {
            map.put(name, value);
        }

    }


    /**
     * <p>Add a nested object literal array value to this object literal,
     * unless the value is null, in which case it is ignored.</p>
     *
     * @param name Name of this field
     * @param value Value of this field
     */
    public void add(String name, ObjectLiteralArray value) {

        if (value != null) {
            map.put(name, value);
        }

    }


    /**
     * <p>Add a string value to this object literal, unless
     * the value is null, in which case it is ignored.</p>
     *
     * @param name Name of this field
     * @param value Value of this field
     */
    public void add(String name, String value) {

        if (value != null) {
            map.put(name, value);
        }

    }


    /**
     * <p>Return an object literal string containing all values that
     * have been added.</p>
     */
    public String toString() {

        StringBuffer sb = new StringBuffer("{");
        Iterator entries = map.entrySet().iterator();
        boolean first = true;
        while (entries.hasNext()) {
            Map.Entry entry = (Map.Entry) entries.next();
            if (first) {
                first = false;
            } else {
                sb.append(',');
            }
            sb.append((String) entry.getKey());
            sb.append(":");
            Object v = entry.getValue();
            if (v == null) {
                continue;
            } else if (v instanceof Boolean) {
                if (((Boolean) v).booleanValue()) {
                    sb.append("true");
                } else {
                    sb.append("false");
                }
            } else if (v instanceof Integer) {
                sb.append(((Integer) v).toString());
            } else if (v instanceof ObjectLiteral) {
                sb.append(((ObjectLiteral) v).toString());
            } else if (v instanceof ObjectLiteralArray) {
                sb.append(((ObjectLiteralArray) v).toString());
            } else if (v instanceof String) {
                sb.append("'");
                sb.append((String) v);
                sb.append("'");
            } else {
                sb.append("'");
                sb.append(v.toString());
                sb.append("'");
            }
        }
        sb.append('}');
        return sb.toString();

    }


}
