/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * <p>Helper class to assist in creating strings in JavaScript
 * <em>object literal</em> notation.  Simply call an <code>add()</code>
 * method for each value to be included, then call <code>toString()</code>
 * to return the object literal array string (including the surrounding
 * square brackets).</p>
 *
 * <p><strong>IMPLEMENTATION NOTE</strong> - It is <strong>NOT</strong>
 * safe to call an <code>add()</code> method on more than one thread
 * at the same time.</p>
 */
public class ObjectLiteralArray {
    

    // ------------------------------------------------------ Instance Variables


    /**
     * <p><code>List</code> containing our values.</p>
     */
    private List list = new ArrayList();


    // ---------------------------------------------------------- Public Methods


    /**
     * <p>Add a boolean value to this object literal array.</p>
     *
     * @param value Value of this field
     */
    public void add(String name, boolean value) {

        list.add((value == true) ? Boolean.TRUE : Boolean.FALSE);

    }


    /**
     * <p>Add an integer value to this object literal array.</p>
     *
     * @param value Value of this field
     */
    public void add(int value) {

        list.add(new Integer(value));

    }


    /**
     * <p>Add a nested object literal value to this object literal array,
     * unless the value is null, in which case it is ignored.</p>
     *
     * @param value Value of this field
     */
    public void add(ObjectLiteral value) {

        if (value != null) {
            list.add(value);
        }

    }


    /**
     * <p>Add a nested object literal array value to this object literal array,
     * unless the value is null, in which case it is ignored.</p>
     *
     * @param value Value of this field
     */
    public void add(ObjectLiteralArray value) {

        if (value != null) {
            list.add(value);
        }

    }


    /**
     * <p>Add a string value to this object literal array, unless
     * the value is null, in which case it is ignored.</p>
     *
     * @param value Value of this field
     */
    public void add(String value) {

        if (value != null) {
            list.add(value);
        }

    }


    /**
     * <p>Return an object literal array string containing all values that
     * have been added.</p>
     */
    public String toString() {

        StringBuffer sb = new StringBuffer("[");
        Iterator entries = list.iterator();
        boolean first = true;
        while (entries.hasNext()) {
            Object entry = entries.next();
            if (first) {
                first = false;
            } else {
                sb.append(',');
            }
            if (entry == null) {
                continue;
            } else if (entry instanceof Boolean) {
                if (((Boolean) entry).booleanValue()) {
                    sb.append("true");
                } else {
                    sb.append("false");
                }
            } else if (entry instanceof Integer) {
                sb.append(((Integer) entry).toString());
            } else if (entry instanceof ObjectLiteral) {
                sb.append(((ObjectLiteral) entry).toString());
            } else if (entry instanceof ObjectLiteralArray) {
                sb.append(((ObjectLiteralArray) entry).toString());
            } else if (entry instanceof String) {
                sb.append("'");
                sb.append((String) entry);
                sb.append("'");
            } else {
                sb.append("'");
                sb.append(entry.toString());
                sb.append("'");
            }
        }
        sb.append(']');
        return sb.toString();

    }


}
