/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.SpryAccordion;
import com.sun.jmaki.faces.component.SpryAccordionSection;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Spry Accordion widget.</p>
 */
public final class SpryAccordionRenderer extends AbstractRenderer {
    

    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the beginning design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designBegin(FacesContext context, UIComponent component,
                               ResponseWriter writer) throws IOException {

        writer.startElement("div", component);
        writer.writeAttribute("id", component.getClientId(context), "id");
        writer.writeAttribute("class", "Accordion", null);
        writer.write("\n");

    }


    /**
     * <p>Render the children design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designChildren(FacesContext context, UIComponent component,
                                  ResponseWriter writer) throws IOException {

        SpryAccordion widget = (SpryAccordion) component;
        int current = widget.getCurrent();
        List kids = widget.getChildren();
        for (int i = 0; i < kids.size(); i++) {

            // Only process children of the correct type
            if (!(kids.get(i) instanceof SpryAccordionSection)) {
                continue;
            }
            SpryAccordionSection section = (SpryAccordionSection) kids.get(i);

            // Begin an element surrounding the entire section
            writer.startElement("div", section);
            writer.writeAttribute("class", "AccordionPanel", null);
            writer.write("\n");

            // Render the section header for this section
            writer.startElement("div", section);
            String gradientClass = "AquaAccordionTab";
            String gradient = widget.getGradient();
            if ("blue".equals(gradient)) {
                gradientClass = "BlueAccordionTab";
            } else if ("green".equals(gradient)) {
                gradientClass = "GreenAccordionTab";
            } else if ("gray".equals(gradient)) {
                gradientClass = "GrayAccordionTab";
            }
            writer.writeAttribute("class", gradientClass, null);
            writer.writeText(valueString(context, section), "value");
            writer.endElement("div");
            writer.write("\n");

            // Render the section content for this section
            // if we are the section that is current, or this is really runtime
            if ((i == current) || !Beans.isDesignTime()) {
                writer.startElement("div", section);
                writer.writeAttribute("class", "AccordionPanelContent", null);
                writer.write("\n");
                Iterator grandkids = section.getChildren().iterator();
                while (grandkids.hasNext()) {
                    UIComponent grandkid = (UIComponent) grandkids.next();
                    encodeRecursive(context, grandkid);
                }
                writer.write("\n");
                writer.endElement("div");
                writer.write("\n");
            }

            // End the element surrounding the entire section
            writer.endElement("div");
            writer.write("\n");

        }

    }


    /**
     * <p>Render the ending design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.endElement("div");
        writer.write("\n");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        SpryAccordion widget = (SpryAccordion) component;
        ObjectLiteral ol = new ObjectLiteral();
        ol.add("gradient", widget.getGradient());
        ol.add("selectedIndex", widget.getCurrent());
        return ol.toString();

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "spry.accordion";

    }


    /**
     * <p>Begin intercepting rendered output from this component and
     * all of its children, then render the same conent we would during
     * design time.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeBegin(FacesContext context, UIComponent component,
                                ResponseWriter writer) throws IOException {

        InterceptingResponseWriter irw =
                new InterceptingResponseWriter(writer);
        context.setResponseWriter(irw);
        designBegin(context, component, irw);

    }


    /**
     * <p>Render the children runtime representation of this widget,
     * duplicating the content we render at design time.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeChildren(FacesContext context, UIComponent component,
                                   ResponseWriter writer) throws IOException {

        designChildren(context, component, writer);

    }


    /**
     * <p>Render the ending runtime representation of this widget,
     * by duplicating the content we render at design time and
     * then invoking jMaki in the usual way.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeEnd(FacesContext context, UIComponent component,
                              ResponseWriter writer) throws IOException {

        designEnd(context, component, writer);
        super.runtimeEnd(context, component, writer);

    }


    // --------------------------------------------------------- Private Methods


}
