/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.YahooTabbedView;
import com.sun.jmaki.faces.component.YahooTabbedViewTab;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Yahoo Tabbed View widget.</p>
 */
public final class YahooTabbedViewRenderer extends AbstractRenderer {
    

    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the beginning design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designBegin(FacesContext context, UIComponent component,
                               ResponseWriter writer) throws IOException {

        writer.startElement("div", component);
        writer.writeAttribute("id", component.getClientId(context), "id");
        writer.write("\n");

    }


    /**
     * <p>Render the children design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designChildren(FacesContext context, UIComponent component,
                                  ResponseWriter writer) throws IOException {

        YahooTabbedView widget = (YahooTabbedView) component;
        int current = widget.getCurrent();
        List kids = widget.getChildren();

        // First pass renders the tab labels
        for (int i = 0; i < kids.size(); i++) {

            // Only process children of the correct type
            if (!(kids.get(i) instanceof YahooTabbedViewTab)) {
                continue;
            }
            YahooTabbedViewTab tab = (YahooTabbedViewTab) kids.get(i);

            // Write the label for this tab
            writer.startElement("span", widget);
            if (i == current) {
                writer.writeAttribute("style", "background-color: white; border-width: 1px; border-color: gray;", null);
            } else {
                writer.writeAttribute("style", "background-color: gray; border-width: 1px; border-color: black;", null);
            }
            writer.writeText(" ", null);
            writer.startElement("a", widget);
            writer.writeAttribute("href", "#" + widget.getClientId(context) + "_" + i, null);
            writer.startElement("em", widget);
            writer.writeText(tab.getValue(), null);
            writer.endElement("em");
            writer.endElement("a");
            writer.writeText(" ", null);
            writer.endElement("span");
            writer.write("\n");

        }
        writer.write("\n");

        // Second pass renders the tab content for the selected tab
        writer.startElement("div", widget);
        writer.writeAttribute("style", "height: 100%; width: 100%; border-width: 1px; border-color: gray", null);
        writer.write("\n");
        for (int i = 0; i < kids.size(); i++) {

            // Only process the currently selected tab
            if (i != current) {
                continue;
            }

            // Only process children of the correct type
            if (!(kids.get(i) instanceof YahooTabbedViewTab)) {
                continue;
            }
            YahooTabbedViewTab tab = (YahooTabbedViewTab) kids.get(i);

            // Write the content for this tab
            writer.startElement("div", tab);
            writer.writeAttribute("style", "height: 100%; width: 100%", null);
            writer.writeAttribute("id", widget.getClientId(context) + "_" + i, null);
            writer.write("\n");
            Iterator grandkids = tab.getChildren().iterator();
            while (grandkids.hasNext()) {
                UIComponent grandkid = (UIComponent) grandkids.next();
                encodeRecursive(context, grandkid);
            }
            writer.write("\n");
            writer.endElement("div");
            writer.write("\n");

        }
        writer.endElement("div");
        writer.write("\n");

    }


    /**
     * <p>Render the ending design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.endElement("div");
        writer.write("\n");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        YahooTabbedView widget = (YahooTabbedView) component;
        ObjectLiteral ol = new ObjectLiteral();
        ol.add("selectedIndex", widget.getCurrent());
        return ol.toString();

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "yahoo.tabbedview";

    }


    /**
     * <p>Begin intercepting rendered output from this component and
     * all of its children, then render the same conent we would during
     * design time.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeBegin(FacesContext context, UIComponent component,
                                ResponseWriter writer) throws IOException {

        InterceptingResponseWriter irw =
                new InterceptingResponseWriter(writer);
        context.setResponseWriter(irw);

        writer.startElement("div", component);
        writer.writeAttribute("id", component.getClientId(context), "id");
        writer.writeAttribute("class", "yui-navset", null);
        writer.write("\n");


    }


    /**
     * <p>Render the children runtime representation of this widget,
     * duplicating the content we render at design time.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeChildren(FacesContext context, UIComponent component,
                                   ResponseWriter writer) throws IOException {

        YahooTabbedView widget = (YahooTabbedView) component;
        int current = widget.getCurrent();
        List kids = widget.getChildren();

        // First pass renders the tab labels
        writer.startElement("ul", widget);
        writer.writeAttribute("class", "yui-nav", null);
        writer.write("\n");
        for (int i = 0; i < kids.size(); i++) {

            // Only process children of the correct type
            if (!(kids.get(i) instanceof YahooTabbedViewTab)) {
                continue;
            }
            YahooTabbedViewTab tab = (YahooTabbedViewTab) kids.get(i);

            // Write the label for this tab
            writer.startElement("li", tab);
            if (i == widget.getCurrent()) {
                writer.writeAttribute("class", "selected", null);
            } else if (tab.isDisabled()) {
                writer.writeAttribute("class", "disabled", "disabled");
            }
            writer.startElement("a", tab);
            writer.writeAttribute("href", "#" + widget.getClientId(context) + "_" + i, null);
            writer.startElement("em", tab);
            writer.writeText(tab.getValue(), null);
            writer.endElement("em");
            writer.endElement("a");
            writer.endElement("li");
            writer.write("\n");

        }
        writer.endElement("ul");
        writer.write("\n");

        // Second pass renders the tab content
        writer.startElement("div", widget);
        writer.writeAttribute("class", "yui-content", null);
        writer.write("\n");
        for (int i = 0; i < kids.size(); i++) {

            // Only process children of the correct type
            if (!(kids.get(i) instanceof YahooTabbedViewTab)) {
                continue;
            }
            YahooTabbedViewTab tab = (YahooTabbedViewTab) kids.get(i);

            // Write the content for this tab
            writer.startElement("div", tab);
            writer.writeAttribute("id", widget.getClientId(context) + "_" + i, null);
            writer.write("\n");
            Iterator grandkids = tab.getChildren().iterator();
            while (grandkids.hasNext()) {
                UIComponent grandkid = (UIComponent) grandkids.next();
                encodeRecursive(context, grandkid);
            }
            writer.write("\n");
            writer.endElement("div");
            writer.write("\n");

        }
        writer.endElement("div");
        writer.write("\n");

    }


    /**
     * <p>Render the ending runtime representation of this widget,
     * by duplicating the content we render at design time and
     * then invoking jMaki in the usual way.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void runtimeEnd(FacesContext context, UIComponent component,
                              ResponseWriter writer) throws IOException {

        writer.endElement("div");
        writer.write("\n");

        super.runtimeEnd(context, component, writer);

    }


    // --------------------------------------------------------- Private Methods


}
