/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.YahooTreeView;
import com.sun.jmaki.faces.component.YahooTreeViewNode;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Yahoo Tree View widget.</p>
 */
public final class YahooTreeViewRenderer extends AbstractRenderer {
    

    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the beginning design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designBegin(FacesContext context, UIComponent component,
                               ResponseWriter writer) throws IOException {

        writer.startElement("p", component);
        writer.writeText(valueString(context, component), "value");
        writer.endElement("p");
        writer.write("\n");
        writer.startElement("ul", component);
        writer.writeAttribute("id", component.getClientId(context), "id");
        writer.write("\n");

    }


    /**
     * <p>Render the children design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designChildren(FacesContext context, UIComponent component,
                                  ResponseWriter writer) throws IOException {

        YahooTreeView widget = (YahooTreeView) component;
        if (widget.isExpanded()) {
            List kids = widget.getChildren();
            designDescendants(context, kids, writer);
        }

    }


    /**
     * <p>Render the ending design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.endElement("ul");
        writer.write("\n");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        return null;

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "yahoo.tree";

    }


    /**
     * <p>Return a data structure that will be used to dynamically create the
     * tree on the client side.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramValue(FacesContext context, UIComponent component) {

        YahooTreeView widget = (YahooTreeView) component;
        ObjectLiteral value = new ObjectLiteral();
        ObjectLiteral root = new ObjectLiteral();

        root.add("title", valueString(context, component));
        root.add("expanded", widget.isExpanded());
        if (widget.getChildCount() > 0) {
            ObjectLiteralArray children =
                    paramChildren(context, widget.getChildren());
            root.add("children", children);
        }
        value.add("root", root);
        if (widget.getCollapseAnim() != null) {
            value.add("collapseAnim", widget.getCollapseAnim());
        }
        if (widget.getExpandAnim() != null) {
            value.add("expandAnim", widget.getExpandAnim());
        }
        return value.toString();

    }


    // --------------------------------------------------------- Private Methods


    /**
     * <p>Recursively render the specified children components.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param kids <code>List</code> of child components to render
     * @param writer <code>ResponseWriter</code> to render to
     *
     * @param IOException if an input/output error occurs
     */
    private void designDescendants(FacesContext context, List kids,
                                   ResponseWriter writer)
      throws IOException {

        for (int i = 0; i < kids.size(); i++) {

            // Only process children of the correct type
            if (!(kids.get(i) instanceof YahooTreeViewNode)) {
                continue;
            }
            YahooTreeViewNode node = (YahooTreeViewNode) kids.get(i);

            // Render this node
            writer.startElement("li", node);
            if (node.getValue() != null) {
                writer.writeText(valueString(context, node), null);
            } else if (node.getHref() != null) {
                writer.write(node.getHref()); // No escaping
            }
            if ((node.getChildCount() > 0) && node.isExpanded()) {
                writer.write("\n");
                writer.startElement("ul", node);
                writer.write("\n");
                designDescendants(context, node.getChildren(), writer);
                writer.endElement("ul");
                writer.write("\n");
            }
            writer.endElement("li");
            writer.write("\n");

        }

    }


    /**
     * <p>Return an {@link ObjectLiteralArray} representing the specified
     * list of child tree nodes.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param kids <code>List</code> of child nodes
     */
    private ObjectLiteralArray paramChildren(FacesContext context, List kids) {

        ObjectLiteral child = null;
        Iterator iter = kids.iterator();
        YahooTreeViewNode kid = null;
        ObjectLiteralArray result = new ObjectLiteralArray();
        
        while (iter.hasNext()) {
            Object next = iter.next();
            if (!(next instanceof YahooTreeViewNode)) {
                continue;
            }
            kid = (YahooTreeViewNode) next;
            child = new ObjectLiteral();
            child.add("label", valueString(context, kid));
            child.add("expanded", kid.isExpanded());
//            if (kid.getHref() != null) {
//                ObjectLiteral onclick = new ObjectLiteral();
//                onclick.add("url", kid.getHref());
//                child.add("onclick", onclick);
//            }
            child.add("href", kid.getHref());
            if (!"ygtvlabel".equals(kid.getLabelStyle())) {
                child.add("style", kid.getLabelStyle());
            }
            child.add("target", kid.getTarget());
            // FIXME - pass "contentStyle"
            // FIXME - pass "content" ???
            // FIXME - pass "multiExpand"
            if (kid.getChildCount() > 0) {
                ObjectLiteralArray children =
                        paramChildren(context, kid.getChildren());
                child.add("children", children);
            }
            result.add(child);
        }
        return result;

    }


}
