/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.tag;

import javax.faces.component.UIComponent;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentTag;

/**
 * <p>Abstract base class for component tags.</p>
 */
public abstract class AbstractComponentTag extends UIComponentTag {
    

    // ------------------------------------------------------ Instance Variables


    private String rendered;
    private String style;
    private String styleClass;


    // -------------------------------------------------- UIComponentTag Methods


    /**
     * <p>Return the component type required for this tag.</p>
     */
    public abstract String getComponentType();


    /**
     * <p>Return the renderer type required for this tag.</p>
     */
    public abstract String getRendererType();


    /**
     * <p>Release resources allocated for this tag instance.</p>
     */
    public void release() {

        super.release();
        this.rendered = null;
        this.style = null;
        this.styleClass = null;

    }


    /**
     * <p>Set properties for the standard attributes common to all
     * components in this library.</p>
     *
     * @param component UIComponent on which to set properties
     */
    protected void setProperties(UIComponent component) {

        super.setProperties(component);
        setBooleanProperty(component, "rendered", this.rendered);
        setStringProperty(component, "style", this.style);
        setStringProperty(component, "styleClass", this.styleClass);

    }
    

    // ------------------------------------------------------- Protected Methods


    /**
     * <p>Set the value of a boolean property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     */
    protected void setBooleanProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            if ("true".equalsIgnoreCase(value) || "yes".equalsIgnoreCase(value)) {
                component.getAttributes().put(name, Boolean.TRUE);
            } else {
                component.getAttributes().put(name, Boolean.FALSE);
            }
        }

    }


    /**
     * <p>Set the value of a Converter property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     *
     * @exception IllegalArgumentException if the value is not an expression
     *  pointing at a Converter instance
     */
    protected void setConverterProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            throw new IllegalArgumentException
              ("Value '" + value + "' is not an expression pointing at a Converter");
        }

    }


    /**
     * <p>Set the value of an integer property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     */
    protected void setIntegerProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            component.getAttributes().put(name, Integer.valueOf(value));
        }

    }


    /**
     * <p>Set the value of a long property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     */
    protected void setLongProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            component.getAttributes().put(name, Long.valueOf(value));
        }

    }


    /**
     * <p>Set the value of an object property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     */
    protected void setObjectProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            component.getAttributes().put(name, value);
        }

    }


    /**
     * <p>Set the value of a string property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     */
    protected void setStringProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            component.getAttributes().put(name, value);
        }

    }


    /**
     * <p>Set the value of a Validator property.</p>
     *
     * @param component UIComponent on which to set a property
     * @param name Name of the property to set
     * @param value Value of the property
     *
     * @exception IllegalArgumentException if the value is not an expression
     *  pointing at a Validator instance
     */
    protected void setValidtorProperty(UIComponent component, String name, String value) {

        if (value == null) {
            return;
        }
        if (isValueReference(value)) {
            ValueBinding vb =
              getFacesContext().getApplication().createValueBinding(value);
            component.setValueBinding(name, vb);
        } else {
            throw new IllegalArgumentException
              ("Value '" + value + "' is not an expression pointing at a Validator");
        }

    }


    // -------------------------------------------------------- Property Setters


    public void setRendered(String rendered) {
        this.rendered = rendered;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }


}
