/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at:
 http://developer.sun.com/berkeley_license.html
 $Id: AjaxCommon.java,v 1.64 2007/03/17 05:54:42 gmurray71 Exp $ */

package com.sun.jmaki;

import org.json.JSONObject;
import org.json.JSONArray;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.logging.*;

/**
 * Common Utilities for both the JSP and JSF AJAX Wrappers.
 *
 * @author Greg Murray
 * @author Ed Burns
 */
public class AjaxCommon {
    
    private static String COUNTER = "jmaki_counter";
    private static String WROTE_SCRIPTS = "jmaki_wrote_script";
    private static String TYPES = "jmaki_types";
    private static String PROPERTIES_OLD = "jmaki_properties_old";
    private static String CHECKED_FOR_UPDATE = "jmaki_checked_for_update";
    private static String WROTE_STYLES = "jmaki_wrote_style";
    private static String WROTE_BOOTSTRAP = "jmaki_wrote_bootstrap";
    private static String WROTE_TYPES = "jmaki_included_types";
    private static String JS_LIBS = "jmaki_js_libs";
    
    private static String CACHE = "jmaki_cache";
    private static String CACHE_TIMES = "jmaki_cache_times";
    private static String DEFAULT_RESOURCE_ROOT = "resources/";
    public static String CLASSPATH_ROOT = "META-INF";
    
    private static String COMPONENT_NAME = "component";
    
    public static final int TEMPLATE = 1;
    public static final int STYLE = 2;
    public static final int SCRIPT = 3;
    
    
    private static Logger logger;
    
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }
    
    /**
     * Mix the two JSONOjbects and always let the prev object win
     *
     */
    public static JSONObject loadTypes(InputStream is, JSONObject prev) {
        JSONObject dConfig = null;
        JSONObject types = null;
        JSONObject apikeys = null;
        JSONObject config = null;
        
        try {
            // start with the prev types and apikeys
            if (prev != null) {
                config = prev;
                if (config.has("types")) {
                    types = config.getJSONObject("types");
                } else {
                    types = new JSONObject();
                }
                if (config.has("apikeys")) {
                    apikeys = config.getJSONObject("apikeys");
                } else {
                    apikeys = new JSONObject();
                }
            } else {
                config = new JSONObject();
                types = new JSONObject();
                apikeys = new JSONObject();
            }
            dConfig = loadJSONObject(is).getJSONObject("config");
            if (dConfig.has("types")) {
                JSONArray sA = dConfig.getJSONArray("types");
                for (int l=0; l < sA.length(); l++) {
                    JSONObject value = sA.getJSONObject(l);
                    String key = value.getString("id");
                    JSONArray styles = new JSONArray();
                    // copy in the styles
                    if (value.has("styles")) {
                        JSONArray nA = value.getJSONArray("styles");
                        for (int il=0; il < nA.length(); il++) {
                            String s = nA.getString(il);
                            styles.put(s);
                        }
                        // initial styles take precedence
                        if (!types.has("styles")) types.put("styles", styles);
                    }
                    // let the existing type takes precedence
                    if (!types.has(key)) types.put(key,value);
                }
            }
            if (dConfig.has("apikeys")) {
                JSONArray nA = dConfig.getJSONArray("apikeys");
                for (int l=0; l < nA.length(); l++) {
                    JSONObject value = nA.getJSONObject(l);
                    String key = value.getString("id");
                    // initial api key takes precedence
                    if (!apikeys.has(key)) {
                        apikeys.put(key,value);
                    }
                }
            }
            config.put("types", types);
            config.put("apikeys", apikeys);
        } catch (Exception ex) {
            getLogger().severe("AjaxCommon error loading config: " + ex);
        }
        return config;
    }
    
    public static JSONObject loadJSONObject(InputStream in) {
        ByteArrayOutputStream out = null;
        try {
            byte[] buffer = new byte[1024];
            int read = 0;
            out = new ByteArrayOutputStream();
            while (true) {
                read = in.read(buffer);
                if (read <= 0) break;
                out.write(buffer, 0, read );
            }
            return new JSONObject(out.toString());
        } catch (Exception e) {
            getLogger().severe("AjaxCommon error reading in json "  + e);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (out != null) {
                    out.flush();
                    out.close();
                }
            } catch (Exception e) {
            }
        }
        return null;
    }
    
    /**
     * Process a tag / component instance using the AjaxContext.
     */
    public static synchronized void doProcess(AjaxContext ajx) {
        
        Writer out = ajx.getWriter();
        
        String resourcesDir = DEFAULT_RESOURCE_ROOT;
        
        String http = ajx.getScheme() + ":";
        String basePath = ajx.getRequestURI();
        String contextPath = ajx.getContextPath();
        
        // working directory is the currrent sub-directory used for
        // searches and is relative to the context root
        // this is an empty string if working in the context root
        String workingDir = "";
        
        JSONObject globalConfig = (JSONObject)ajx.getAttribute(TYPES);
        JSONObject globalTypes = null;
        
        // find the working directory
        // we know the context root is a / so start from postion 1
        int first = basePath.indexOf("/", 1);
        int last = basePath.lastIndexOf("/");
        // find the base path from which to start looking for resources from
        // this is needed if we are working in subdirectories
        if (first != -1 && last != -1) {
            workingDir = basePath.substring(first + 1, last  + 1);
            ajx.getProperties().put("workingDir",workingDir);
        }
        
        String webRoot = ajx.getString("webRoot");
        if (webRoot == null) {
            webRoot = http + "//" + ajx.getServerName();
            if (ajx.getServerPort() != 80) {
                webRoot += ":" + ajx.getServerPort();
            }
            webRoot += contextPath;
            ajx.getProperties().put("webRoot",webRoot);
        }
        
        // check to see if a different resources directory is specified.
        if (ajx.getInitParameter("jmaki-resources") != null) {
            resourcesDir = ajx.getInitParameter("jmaki-resources");
        }
        String xhpMapping = "/xhp";
        // check to see if a different resources directory is specified.
        if (ajx.getInitParameter("jmaki-xhp-mapping") != null) {
            xhpMapping = ajx.getInitParameter("jmaki-xhp-mapping");
        }
        xhpMapping = webRoot + xhpMapping;
        ajx.getProperties().put("xhpMapping", xhpMapping);
        
        if (resourcesDir.endsWith("/")){
            resourcesDir = resourcesDir.substring(0, resourcesDir.length() -1);
        }
        ajx.getProperties().put("resourcesDir", resourcesDir);
        
        boolean checkProps = (ajx.getRequestAttribute(CHECKED_FOR_UPDATE) == null);
        boolean propsUpdated = false;
        // check for an update to the types.json file. This check will only be done one
        // time per-page rendering.
        if (globalConfig == null) {
            propsUpdated = true;
        } else if (checkProps) {
            propsUpdated = AjaxCommon.checkForUpdate("/" + resourcesDir, "/config.json", ajx);
            if (!propsUpdated) {
                propsUpdated = AjaxCommon.checkForUpdate("/WEB-INF/", "config.json", ajx);
            }
            ajx.setRequestAttribute(CHECKED_FOR_UPDATE, new Boolean(true));
        }
        
        if (propsUpdated) {
            try {
                URL url = ajx.getResource("/" + resourcesDir + "/config.json");
                InputStream is = null;
                // First, load the config.json from the application root's resources directory
                if (null != url) {
                    is = url.openConnection().getInputStream();
                    if (is != null) {
                        globalConfig = loadTypes(is, null);
                        is.close();
                    } else {
                        getLogger().severe("AjaxCommon could not find config.json");
                    }
                }
                // Second, load the config.json from the classpath.  First hit wins.
                url = ajx.getClass().getResource("/" + CLASSPATH_ROOT + "/" + resourcesDir + "/config.json");
                if (null != url) {
                    is = url.openConnection().getInputStream();
                    if (is != null) {
                        globalConfig = loadTypes(is, globalConfig);
                        is.close();
                    } else {
                        getLogger().severe("AjaxCommon could not find config.json");
                    }
                }
                ajx.setAttribute(TYPES, globalConfig);
            } catch (IOException e) {
                getLogger().severe("AjaxCommon error loading configuration: " + e);
            }
        }
        try {
            if (globalConfig != null) {
                globalTypes = globalConfig.getJSONObject("types");
            }
        } catch (org.json.JSONException je) {
            getLogger().severe("AjaxCommon error loading global types: " + je);
        }
        
        // generate a uuid
        int counter = 0;
        Integer counterI = (Integer)ajx.getAttribute(COUNTER);
        if (counterI != null) {
            counter = counterI.intValue();
        }
        
        // Default Ids that are incremented
        String uuid =  ajx.getString("uuid");
        String name =  ajx.getString("name");
        if (uuid == null) {
            uuid = name + "" +  counter++;
            // remove the '.' from the namespacing
            uuid = uuid.replace('.', '_');
        }
        ajx.setAttribute(COUNTER, new Integer(counter));
        
        try {
            boolean wroteBootstrapScript = (ajx.getRequestAttribute(WROTE_BOOTSTRAP) != null);
            if (!wroteBootstrapScript) {
                writeBootstrapScript(ajx);
            }
            
            HashMap jsLibs = (HashMap)ajx.getRequestAttribute(JS_LIBS);
            if (jsLibs == null) {
                jsLibs = new HashMap();
                ajx.setRequestAttribute(JS_LIBS,jsLibs);
            }
            HashMap libs = (HashMap)ajx.getRequestAttribute(WROTE_TYPES);
            if (libs == null) {
                libs = new HashMap();
                ajx.setRequestAttribute(WROTE_TYPES,libs);
            }
            
            // Check and see if the style for this component has been written
            HashMap styles = (HashMap)ajx.getRequestAttribute(WROTE_STYLES);
            if (styles == null) {
                styles = new HashMap();
                ajx.setRequestAttribute(WROTE_STYLES,styles);
            }
            
            boolean useClasspath = false;
            // compute this from the script location if automatic otherwise the template location
            // if both are defined as attributes the component doesn't have a base dir and will thus
            // be null
            String compBase = null;
            
            String script = ajx.getString("script");
            if (script == null) {
                String urlString = "/" + workingDir + name.replace('.','/') + "/";
                script = AjaxCommon.getResourceNameByType(urlString, AjaxCommon.SCRIPT, ajx);
                if (script == null) {
                    urlString = "/" + resourcesDir + "/" + name.replace('.','/') + "/";
                    script = AjaxCommon.getResourceNameByType(urlString, AjaxCommon.SCRIPT, ajx);
                    if (script != null) {
                        compBase = "/" + resourcesDir + "/" + name.replace('.', '/') + "/";
                    }
                } else {
                    compBase = "/" + workingDir + name.replace('.','/') + "/";
                }
                // we had a hit on the resourcesDir
                if (script == null) {
                    URL url = checkForResource(CLASSPATH_ROOT + "/" + resourcesDir + "/" +
                            name.replace('.','/') + "/component.js", ajx, false);
                    // found in the classpath
                    if (url != null) {
                        script =  ajx.getString("jsfRoot") + "/" + resourcesDir + "/" + name.replace('.','/') + "/component.js";
                        compBase = "/" + resourcesDir + "/" + name.replace('.', '/') + "/";
                        useClasspath = true;
                    } else {
                        //TODO display an error message
                    }
                }
                script = webRoot + script;
            } else {
                String scriptURL = script;
                if (scriptURL.startsWith("/")) {
                    scriptURL = webRoot + scriptURL;
                } else if (!scriptURL.startsWith("http")) {
                    scriptURL = webRoot + "/" + workingDir + scriptURL;
                }
            }
            
            String type = null;
            
            // full name takes precedent over type in search
            // in other words look for an entry dojo.comboBox first
            if (type == null && (globalTypes != null) && globalTypes.has(name)) {
                type = name;
                // else if a type has not been defined default to the first part of the namespace
                // e.g. dojo.comboBox is assumed to be of type dojo
            } else if (type == null && name.indexOf(".") != -1) {
                int start = name.indexOf(".");
                if (start > 0) {
                    type = name.substring(0, start);
                }
            }
            // Check for the widget.config
            // The widget.config config takes precedence over the global config.json
            JSONObject cfg = getWidgetConfig(ajx, compBase, useClasspath);
            // override the classpath if set by the widget config
            if (ajx.getBoolean("useClasspath")) useClasspath = ajx.getBoolean("useClasspath");
            if (cfg != null) {
                try {
                    JSONObject jType = cfg.getJSONObject("type");
                    String sType = jType.getString("id");
                    if (!libs.containsKey(sType)) {
                        String wr = webRoot;
                            if (useClasspath &&  ajx.getString("jsfRoot")  != null) {
                                wr += ajx.getString("jsfRoot");
                        }                    
                        // get the JSONObject for the specific type
                        // write out the styles first as the scripts might depend on them
                        if (jType.has("styles")) {
                            writeStyles(out, jType, styles, http, wr,  compBase);
                        }
                        // write out all the libs and dependencies
                        writeLibs(out, jType, cfg, jsLibs, http, wr, compBase);
                        libs.put(sType, new Boolean(true));
                    }
                } catch (org.json.JSONException jx) {
                    getLogger().severe("AjaxCommon JSON Error: " + jx);
                }
                // check and see if the libraries for the type  have been written to this current page
            } else if (globalTypes != null) {
                // split of the types (in case there is more than one comman separated)
                try {
                    if (globalTypes.has(type)) {
                        if (!libs.containsKey(type)) {
                            // get the JSONObject for the specific type
                            JSONObject jType = globalTypes.getJSONObject(type);
                            String wr = webRoot;
                            // write out the styles first as the scripts might depend on them
                            if (useClasspath &&  ajx.getString("jsfRoot")  != null) {
                                wr += ajx.getString("jsfRoot");
                            }
                            if (jType.has("styles")) {
                                writeStyles(out, jType, styles, http, wr, null);
                            }
                            // write out all the libs and dependencies
                            writeLibs(out, jType, globalConfig, jsLibs, http, wr, null);
                            libs.put(type, new Boolean(true));
                        }
                    }
                } catch (org.json.JSONException jsx) {
                    getLogger().severe("AjaxCommon error processing types"  + jsx);
                }
            }
            
            String style = ajx.getString("style");
            boolean wroteStyle =  (styles.get(name) != null);
            if (!wroteStyle) {
                if (style == null) {
                    String styleURL = null;
                    if (!useClasspath) {
                        styleURL = AjaxCommon.getResourceNameByType(compBase, AjaxCommon.STYLE, ajx);
                    } else {
                        URL url = checkForResource(CLASSPATH_ROOT + compBase + "component.css", ajx, false);
                        if (url != null) {
                            if (ajx.getString("jsfRoot")  != null) {
                                styleURL =  ajx.getString("jsfRoot") + compBase + "component.css";
                            } else {
                                styleURL = compBase + "component.css";
                            }
                        }
                    }
                    if (styleURL != null) {
                        style = webRoot + styleURL;
                        out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + style + "\"></link>\n");
                    }
                } else if (style != null) {
                    String styleURL = style;
                    if (styleURL.startsWith("/")) {
                        styleURL = webRoot + styleURL;
                    } else if (!styleURL.startsWith("http")) {
                        styleURL = webRoot + "/" + workingDir + styleURL;
                    }
                    out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + styleURL + "\"></link>\n");
                }
                styles.put(name, new Boolean(true));
            }
            
            boolean wroteScript =  (libs.get(name) != null);
            if (script != null && !jsLibs.containsKey(script)) {
                out.write("<script type=\"text/javascript\" src=\"" + script +  "\"></script>\n");
                jsLibs.put(script, new Boolean(true));
            }
            
            StringBuffer template = ajx.getTemplate();
            String templateURL = "";
            
            // use the body content of the tag if provided otherwise look for the template resource file
            // first as the attribute (for over-rides) and secondly in the widget directory.
            if (template == null) {
                templateURL = ajx.getString("template");
                if (templateURL != null) {
                    // look in the current directory if the resource name does not start with a /
                    if (!templateURL.startsWith(http)  &&
                            !templateURL.startsWith("/")) {
                        if ("".equals(workingDir)) templateURL = webRoot + "/" + templateURL;
                        else templateURL = webRoot + "/" + resourcesDir + "/" + templateURL;
                        // start in the web root otherwise
                    } else if (templateURL.startsWith("/")) {
                        templateURL = webRoot + "/" + templateURL;
                    }
                    try {
                        URL turl = new URL(templateURL);
                        template = getResource(turl.openStream());
                    } catch (Exception ex) {
                        getLogger().severe("AjaxCommon URL error loading template  + : "  + templateURL);
                    }
                    
                } else {
                    if (!useClasspath) {
                        templateURL = AjaxCommon.getResourceNameByType(compBase, AjaxCommon.TEMPLATE, ajx);
                        if (templateURL != null) {
                            StringBuffer buff = AjaxCommon.getResource(templateURL, ajx);
                            // do this to prevent a NPE
                            if (buff != null) template = new StringBuffer(buff.toString());
                        }
                    } else {
                        URL url = checkForResource(CLASSPATH_ROOT + compBase + "component.htm", ajx, false);
                        if (url == null) {
                            url = checkForResource(CLASSPATH_ROOT + compBase + "component.html", ajx, false);
                        }
                        if (url != null) {
                            StringBuffer buff = AjaxCommon.getResource(url.openStream());
                            // do this to prevent a NPE
                            if (buff != null) template = new StringBuffer(buff.toString());
                        }
                    }
                    
                    // set the component base to the template directory if it could not be
                    // computed from the script location
                    if (templateURL != null && compBase == null) {
                        if (!templateURL.startsWith("/")) {
                            templateURL = webRoot + "/" + workingDir + "/" + templateURL;
                            // start in the web root otherwise
                        } else {
                            templateURL = webRoot + templateURL;
                        }
                    }
                }
            }
            // remove the trailing slash
            String widgetDir = webRoot;
            if (useClasspath) widgetDir += ajx.getString("jsfRoot");
            if (widgetDir != null &&
                    widgetDir.endsWith("/") && widgetDir.length() > 0) widgetDir = widgetDir.substring(0,widgetDir.length() -1);
            
            if (compBase != null && compBase.length() > 0) widgetDir += compBase.substring(0, compBase.length() -1);
            
            String service = ajx.getString("service");
            if (service != null && !service.startsWith("http") && service.startsWith("/")) {
                service = webRoot + service;
            } else if (service != null && !service.startsWith("http")) {
                service = webRoot + "/" + workingDir + service;
            }
            String value = ajx.getString("value");
            
            // now write the template out with the string replacments
            if (template == null) {
                out.write("<h2>Error</h2> <p>Unable to locate the template file for widget " + name  +
                        ". Please make sure the widget is available.</p>");
                return;
            } else {
                // now that we have everything do the evaluation
                AjaxCommon.replace(template, "${uuid}", uuid);
                // default service URL to the name
                if (service != null) {
                    AjaxCommon.replace(template, "${service}", service);
                }
                if (value != null && !value.startsWith("@{")) {
                    AjaxCommon.replace(template, "${value}", value);
                    // replace value bindings in templates with blanks
                } else if (value != null && value.startsWith("@{")) {
                    AjaxCommon.replace(template, "${value}", "");
                }
                AjaxCommon.replace(template, "${contextPath}", webRoot);
                AjaxCommon.replace(template, "${widgetDir}", widgetDir);
                out.write(template.toString());
            }
            
            // generate the object literal to configure the widget
            HashMap props = new HashMap();
            props.put("uuid", uuid);
            props.put("name", name);
            props.put("widgetDir", widgetDir);
            props.put("script", script);
            String selected = ajx.getString("selected");
            if (selected != null){
                props.put("selected", selected);
            }
            if (service != null) {
                props.put("service", service);
            }
            String valueCallback = ajx.getString("valueCallback");
            if (valueCallback != null) {
                props.put("valueCallback", valueCallback);
            }
            if (value != null) {
                props.put("value", value);
            }
            String args = ajx.getString("args");
            if (args != null) {
                props.put("args", args);
            }
            out.write("<script type=\"text/javascript\">\n");
            out.write(" jmaki.addWidget(" + AjaxCommon.createJavaScriptObject(props).toString() + ");");
            out.write("</script>\n");
            props = null;
            
        } catch (java.io.IOException iox) {
            getLogger().severe("AjaxCommon IO Error: " + iox);
        }
    }
    
    private static void writeBootstrapScript(AjaxContext ajx ) {
        Writer out = ajx.getWriter();
        //ServletRequest request = ajx.getRequest();
        String resourcesDir = ajx.getString("resourcesDir");
        String webRoot = ajx.getString("webRoot");
        boolean embedBootstrap = false;
        String urlString = null;
        // check to see if the jmaki.js was included in the web app
        try {
            urlString = "/jmaki.js";
            URL url = ajx.getResource(urlString);
            if (url == null) {
                urlString = "/" + resourcesDir + "/jmaki.js";
                url = ajx.getResource(urlString);
                // if we are JSF we can load from the classpath
            }
            if (url == null) {
                if (checkForResource(CLASSPATH_ROOT + "/" + resourcesDir + "/jmaki.js", ajx, false) != null) {
                    urlString = ajx.getString("jsfRoot") + "/" + resourcesDir + "/jmaki.js";
                } else {
                    embedBootstrap = true;
                }
            }
            
            if (embedBootstrap) {
                StringBuffer bootstrapper = AjaxCommon.getResource("jmaki.js", ajx);
                if (bootstrapper != null) {
                    out.write("<script type=\"text/javascript\">" + bootstrapper.toString() + "</script>\n");
                    out.write("<script type=\"text/javascript\">jmaki.webRoot='" + webRoot + "';");
                    out.write("jmaki.resourcesRoot ='/" + resourcesDir + "';jmaki.xhp ='" + ajx.getString("xhpMapping") + "';</script>\n");
                } else {
                    out.write("<h2>Error</h2> <p>Unable to locate the jmaki.js in the component jar or web root. Please check to make sure the file is available.</p>");
                    getLogger().severe("AjaxTag error: Unable to locate the jmaki.js in the component jar or web root.");
                }
                ajx.setRequestAttribute(WROTE_BOOTSTRAP, new Boolean(true));
            } else {
                out.write("<script type=\"text/javascript\" src=\"" + webRoot + urlString + "\"></script>\n");
                out.write("<script type=\"text/javascript\">jmaki.webRoot='" + webRoot + "';");
                out.write("jmaki.resourcesRoot ='/" + resourcesDir + "';jmaki.xhp ='" + ajx.getString("xhpMapping") + "';</script>\n");
                ajx.setRequestAttribute(WROTE_BOOTSTRAP, new Boolean(true));
            }
        } catch (IOException e) {
            // don't do anything
        }
        
    }
    
    /*
     * Load the widget.config. There is one level of caching done on the content of the file and that is only
     * re-loaded if the file has been updated. The JSON text is re-parsed on every widget call.
     * To optimize it would be best to do some more chaching here which will be a memory cost on the server.
     *
     */
    private static JSONObject getWidgetConfig(AjaxContext ajx, String compBase, boolean useClasspath) {
        
        JSONObject cfg = null;
        JSONObject wf = null;
        try {
            URL url = null;
            String urlString = null;
            if (!useClasspath) {
                url = ajx.getResource(compBase  + "widget.json");
                if (url != null) urlString = compBase  + "widget.json";
            } else {
                url = ajx.getClass().getResource("/" + CLASSPATH_ROOT + compBase + "widget.json");
                if (url != null) urlString = "/" + CLASSPATH_ROOT + compBase + "widget.json";
                // set loading from classpath to true
                ajx.getProperties().put("useClasspath", new Boolean(true));
            }
            if (urlString == null) return null;
            // load the widget.json in such a way that it is cached.
            StringBuffer buff = getResource(urlString,true, null, ajx);
            if (buff == null) return null;
            try {
                wf = new JSONObject(buff.toString());
            } catch (org.json.JSONException jx) {
                getLogger().severe("AjaxCommon JSON Error: " + jx);
            }
            
            if (wf != null &&  wf.has("config")) {
                cfg = wf.getJSONObject("config");
            } else {
                return null;
            }
            // reformat the apikeys to be friendly to searches
            if (cfg.has("apikeys")) {
                JSONArray nA = cfg.getJSONArray("apikeys");
                JSONObject apikeys = new JSONObject();
                for (int l=0; l < nA.length(); l++) {
                    JSONObject value = nA.getJSONObject(l);
                    String key = value.getString("id");
                    apikeys.put(key,value);
                }
                cfg.put("apikeys", apikeys);
            }
        } catch (org.json.JSONException jx) {
            getLogger().severe("AjaxCommon JSON Error: " + jx);
        } catch (IOException e) {
            getLogger().severe("AjaxCommon error loading configuration: " + e);
        }
        return cfg;
    }
    
    
    /*
     * Write out the styles first as the scripts might depend on them
     */
    private static void writeStyles(Writer out, JSONObject jType, HashMap styles, String http,
            String webRoot, String widgetRoot)
    throws org.json.JSONException, java.io.IOException {
        
        JSONArray lA = jType.getJSONArray("styles");
        for (int l = 0; l < lA.length(); l++) {
            String lStyle = lA.getString(l);
            if (lStyle != null) {
                if (!lStyle.startsWith(http) && widgetRoot == null) {
                    lStyle = webRoot + lStyle;
                } else if (!lStyle.startsWith(http)) {
                    if (lStyle.startsWith("/")) {
                        lStyle = webRoot + lStyle;
                    } else {
                        lStyle = webRoot + widgetRoot + lStyle;
                    }
                }
                out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\""
                        + lStyle + "\"></link>\n");
                // put the style in the global style map to make sure it doesn't get written again
                styles.put(lStyle, new Boolean(true));
            }
        }
    }
    
    /*
     *  Write library dependencies with API keys
     */
    private static void writeLibs(Writer out, JSONObject jType, JSONObject cfg,
            HashMap jsLibs, String http, String webRoot, String widgetRoot)
            throws org.json.JSONException, java.io.IOException {
        JSONArray sA = jType.getJSONArray("libs");
        if (sA == null) return;
        for (int l = 0; l < sA.length(); l++) {
            String library = "";
            String jlib = sA.getString(l);
            // check if we need to write an API key
            if (jType.has("apikey")) {
                String apiKey = jType.getString("apikey");
                JSONObject globalAPIKeys = cfg.getJSONObject("apikeys");
                if (globalAPIKeys.has(apiKey)) {
                    JSONObject targetKey = globalAPIKeys.getJSONObject(apiKey);
                    // get the key set
                    JSONArray targetKeys = targetKey.getJSONArray("keys");
                    // check each API against the current URL to select the correct key
                    for (int al = 0; al < targetKeys.length(); al++) {
                        JSONObject apiJSONObject = targetKeys.getJSONObject(al);
                        String targetURL = apiJSONObject.getString("url");
                        if ((targetURL != null) &&
                                targetURL.equals(webRoot) ||
                                targetURL.equals(webRoot + "/") ||
                                targetURL.equals("*")) {
                            jlib += apiJSONObject.getString("key");
                        }
                    }
                }
            }
            if (jlib != null && jlib.startsWith(http)) {
                library = jlib;
            } else {
                if (widgetRoot == null) {
                    // add a forward slash if it thas been done already
                    if (!jlib.startsWith("/")) {
                        library =  "/";
                    }
                    library = webRoot +  jlib;
                } else {
                    if (jlib.startsWith("/")) {
                        library = webRoot +  jlib;                    
                    } else {
                        library =  webRoot + widgetRoot  + jlib;
                    }
                }
            }
            // write preload library intitializion code
            if (jType.has("preload")) {
                out.write("<script type=\"text/javascript\">" + jType.getString("preload") +  "</script>\n");
            }
            // only write a script src to a js library once
            // if two types define the same JS lib do not duplicate
            if (!jsLibs.containsKey(library)) {
                out.write("<script type=\"text/javascript\" src=\"" + library +  "\"></script>\n");
                jsLibs.put(library, new Boolean(true));
            }
            // write postload library intitializion code
            if (jType.has("postload")) {
                out.write("<script type=\"text/javascript\">" + jType.getString("postload") +  "</script>\n");
            }
        }
        
    }
    
    /**
     * Will first attempt to load the resource from the context root and if it fails will look in the
     * META-INF/resources directory of the component
     */
    public static StringBuffer getResource(String resource, AjaxContext ajx) {
        StringBuffer buffer = getResource(resource, true, DEFAULT_RESOURCE_ROOT, ajx);
        return buffer;
    }
    
    public static boolean checkForUpdate(String resourceRoot, String resource, AjaxContext ajx) {
        URL url = AjaxCommon.class.getResource(resourceRoot + resource);
        URLConnection con;
        try {
            if (url == null) {
                url = ajx.getResource(resourceRoot + resource);
            }
            if (url == null) return false ;
            
            con = url.openConnection();
            HashMap cache = (HashMap)ajx.getAttribute(CACHE);
            HashMap cacheTimes = (HashMap)ajx.getAttribute(CACHE_TIMES);
            if (cache == null) {
                cache = new HashMap();
                cacheTimes = new HashMap();
                ajx.setAttribute(CACHE, cache);
                ajx.setAttribute(CACHE_TIMES, cacheTimes);
            }
            long lastModified = con.getLastModified();
            long cacheModified = 0;
            if (cacheTimes.get(resourceRoot + resource) != null) {
                cacheModified = ((Long)cacheTimes.get(resourceRoot + resource)).longValue();
            }
            if (cacheModified < lastModified) {
                synchronized(cacheTimes) {
                    cacheTimes.put(resourceRoot + resource, new Long(lastModified));
                    return true;
                }
            }
        } catch (IOException e) {
            getLogger().severe("AjaxCommon IO error loading properties: " + e);
        } catch (Exception ex) {
            getLogger().severe("AjaxCommon error loading properties: " + ex);
        }
        return false;
    }
    
    public static StringBuffer getResource(String resource,
            boolean cacheContent, String resourceRoot, AjaxContext ajx) {
        InputStream stream = null;
        try {
            URL url = ajx.getResource(resource);
            if (url == null) {
                url = AjaxCommon.class.getResource(resource);
            }
            if (url == null) {
                url = AjaxCommon.class.getResource(CLASSPATH_ROOT + resource);
            }
            
            if (url == null)
                return null;
            URLConnection con;
            try{
                
                con = url.openConnection();
                final String fileEncoding = "UTF-8";// TODO get resource encoding
                if (cacheContent) {
                    HashMap cache = (HashMap) ajx.getAttribute(CACHE);
                    HashMap cacheTimes = (HashMap) ajx.getAttribute(CACHE_TIMES);
                    if (cache == null) {
                        cache = new HashMap();
                        cacheTimes = new HashMap();
                        ajx.setAttribute(CACHE, cache);
                        ajx.setAttribute(CACHE_TIMES, cacheTimes);
                    }
                    
                    long lastModified = con.getLastModified();
                    long cacheModified = 0;
                    if (cacheTimes.get(resource) != null) {
                        cacheModified = ((Long) cacheTimes.get(resource))
                        .longValue();
                    }
                    if (cacheModified < lastModified) {
                        stream =con.getInputStream();
                        StringBuffer buffer = getResource(stream,
                                fileEncoding);
                        synchronized (cacheTimes) {
                            cacheTimes.put(resource, new Long(lastModified));
                        }
                        synchronized (cache) {
                            cache.put(resource, buffer);
                        }
                        return buffer;
                    } else {
                        return (StringBuffer) cache.get(resource);
                    }
                } else {
                    stream =con.getInputStream();
                    return getResource(stream, fileEncoding);
                }
                
            }finally{
                if( stream != null ){
                    stream.close();
                }
            }
        } catch (Exception e) {
            getLogger().info("AjaxCommon : Could not load " + resource);
        }
        return null;
    }
    
    
    /**
     * This function will find the URL to a specified resource
     *
     */
    public static String getResourceNameByType(String compBase, int type, AjaxContext ajx) {
        if (compBase == null) {
            getLogger().info("AjaxCommon:getResourceByType: compBase is null.");
            return null;
        }
        String extension = null;
        switch (type) {
            case STYLE : extension = ".css"; break;
            case SCRIPT : extension = ".js"; break;
            case TEMPLATE : extension = ".htm"; break;
        }
        if (extension == null) {
            getLogger().severe("AjaxCommon:getResourceByType: type " + type +  " not supported.");
        }
        
        String urlName = compBase + COMPONENT_NAME +  extension;
        // check the currrent directory
        URL url = checkForResource(urlName, ajx, true);
        if (url == null && type == TEMPLATE) {
            urlName += "l";
            // check for component.html
            url = checkForResource(urlName, ajx, true);
        }
        if (url != null) {
            return urlName;
        } else {
            return null;
        }
    }
    
    public static URL checkForResource(String resource, AjaxContext ajx, boolean fromWeb) {
        URL url = null;
        try {
            if (!fromWeb) {
                return  AjaxCommon.class.getClassLoader().getResource(resource);
            } else {
                return ajx.getResource(resource);
            }
        } catch (MalformedURLException mue) {
            // do nothing here as we are just fishing for resources
        }
        return null;
    }
    
    public static StringBuffer getResource(InputStream stream) {
        return getResource(stream,null);
    }
    
    public static StringBuffer getResource(InputStream stream, String encoding) {
        StringBuffer buffer = new StringBuffer();
        BufferedReader bufReader = null;
        String curLine = null;
        try {
            try {
                final InputStreamReader inputStreamReader = (  encoding == null ) ?new InputStreamReader(stream) : new InputStreamReader(stream,
                        encoding);
                bufReader = new BufferedReader(inputStreamReader);
                while (null != (curLine = bufReader.readLine())) {
                    buffer.append(curLine + "\n");
                }
            } finally {
                if (bufReader != null) {
                    bufReader.close();
                }
            }
        } catch (Exception e) {
            getLogger()
            .severe("AjaxCommon:loadResource from stream error:" + e);
        }
        return buffer;
    }
    
    public static void replace(StringBuffer buff, String target, String replacement) {
        if (buff == null || target == null || replacement == null) {
            return;
        }
        int index = 0;
        while (index < buff.length()) {
            index = buff.indexOf(target);
            if (index == -1) {
                break;
            }
            buff.replace(index, index +  target.length(), replacement);
            index += replacement.length() + 1;
        }
    }
    
    /**
     *   Convert a Map to a JavaScript object literal StringBuffer
     *
     */
    public static StringBuffer createJavaScriptObject(Map p) {
        return createJavaScriptObject(p, false);
    }
    
    /**
     *  Convert a Map to a JavaScript object literal or JSON StringBuffer
     *  Values that are Maps are considered sub-objects and this method is
     *  recursively called on those objects.
     *
     */
    public static StringBuffer createJavaScriptObject(Map p, boolean json) {
        StringBuffer buff = new StringBuffer("{");
        Iterator it = p.entrySet().iterator();
        
        while (it.hasNext()) {
            String key = (String)((Map.Entry)it.next()).getKey();
            Object val = p.get(key);
            // a value is a map it is an embedded object so call this method recursively
            if ((val != null) && val instanceof java.util.Map) {
                val = createJavaScriptObject((Map)val);
            }
            // check if the value is a number
            if (json) {
                buff.append("'" + key + "':");
            } else {
                buff.append(key + ":");
            }
            
            if (val == null) {
                buff.append("null");
            } else if (val instanceof Float ||
                    val instanceof Long ||
                    val instanceof Integer ||
                    val instanceof StringBuffer) {
                buff.append(val.toString());
                // if the value is an object literal or an array as a String
            } else if (val instanceof String &&
                    ((String)val).trim().startsWith("{") ||
                    ((String)val).trim().startsWith("[")
                    ) {
                buff.append(val.toString());
            } else {
                buff.append("'" + val.toString()  + "'");
            }
            if (it.hasNext()) {
                buff.append(",");
            }
        }
        buff.append("}");
        return buff;
    }
}
