package jmaki.runtime;

import java.io.Writer;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.logging.Logger;
import java.net.URL;
import jmaki.runtime.config.GlobalConfig;
import jmaki.runtime.config.Config;
import jmaki.runtime.config.ApiKey;


/**
 * @author Norbert von Truchsess, Greg Murray
 *
 */
public class AjaxCommonRenderer {

	// per request attributes
	
	private AjaxContext ajx;
	private GlobalConfig globalConfig;
	private Config widgetConfig;
	private ResourceManager resourceManager;
	private UriManager uriManager;
	private Writer out;

	private String globalWebRoot;
	private String resourcesDir;
	
	// per widget attributes:
	
	private String name;
	private String uuid;
	private String template;
	private String onLoad; // TODO attribute loLoad is not yet used.
	private String style;
	private String type; // TODO attribute type is not yet used.
	private boolean embedded; // TODO attribute embedded is not yet used.
	private String args;

	private String script;
	private String service;
	private String value;
	private String xhpMapping;
	private String selected;
	private String valueCallback;

	// per widget local values:
	
	private String widgetDir;
	private String absoluteWidgetPath;
	private Iterator libs;
	private Iterator styles;
         
	private boolean isWidgetWebAppResource;
	private boolean isGlobalConfigWebAppResource;
        private URL templateURL = null;
	
	private Map widgetProperties;

	//setter-methods for widget attributes
	public void setName(String name) {this.name = name;}
	public void setUuid(String uuid) {this.uuid = uuid;}
	public void setTemplate(String template) {this.template = template;}
	public void setOnLoad(String onLoad) {this.onLoad = onLoad;}
	public void setStyle(String style) {this.style = style;}
	public void setType(String type) {this.type = type;}
	public void setArgs(String args) {this.args = args;}

	public void setScript(String script) {this.script = script;}
	public void setService(String service) {this.service = service;}
	public void setValue(String value) {this.value = value;}
	public void setSelected(String selected) {this.selected = selected;}
	public void setValueCallback(String valueCallback) {this.valueCallback = valueCallback;}
	public void setEmbedded(boolean embedded) {this.embedded=embedded;}

	public AjaxCommonRenderer(AjaxContext ajx) throws AjaxException {
		this.ajx = ajx;
	}
	
	private void init() throws AjaxException {
		globalConfig = ajx.getConfig();
		resourceManager = ajx.getResourceHandler();

		String widgetPath = resourceManager.getWidgetDir(name);
		String widgetDir = null;

		/**
         * Search for the widget (that is the location where component.js or the script specified is being found)
         *  1. Check the relative directory in the web app
         *  2. Check the resources direcotry in the web app
         *  3. Check the classpath
         */

		//attribute script allows to overwrite default 'component.js'
		if (script == null)
			script = "component.js";

		String tmpScript;
		if (resourceManager.isWebAppResource(tmpScript = IOUtil.resolvePath((widgetDir = ajx.getWorkingDir()+widgetPath),script))) {
			script = tmpScript;
			this.isWidgetWebAppResource=true;
		} else if (resourceManager.isWebAppResource(tmpScript = IOUtil.resolvePath((widgetDir = resourceManager.getWebResourcePath()+widgetPath),script))) {
			script = tmpScript;
			this.isWidgetWebAppResource=true;
		} else if (resourceManager.isClassPathResource(tmpScript = IOUtil.resolvePath((widgetDir = resourceManager.getResourcesDir()+widgetPath),script))) {
			script = tmpScript;
			this.isWidgetWebAppResource=false;
		} else
			throw new WidgetResourceException("unable to locate '"+script+"' in '"+ajx.getWorkingDir()+widgetPath+"', '"+resourceManager.getWebResourcePath()+widgetPath+ "' and ClassPath 'META-INF'"+resourceManager.getResourcesDir()+widgetPath+"'"); 

		uriManager = ajx.getUriManager();
		out = ajx.getWriter();
		
		//uuid
		if (uuid == null) {
			// generate a uuid
			uuid = name + "" + ajx.getUniqueId();
			// remove the '.' from the namespacing
			uuid = uuid.replace('.', '_');
		}
		

		isGlobalConfigWebAppResource = globalConfig.isGlobalConfigWebAppResource();
                // TODO here is an issue if config.json is in both WebApp and Classpath only:
                // gmurray71 : In cases where the globalConfig is in both we have to merge them with the WebApp overridding duplicate properties but mixing
                // glue from both and types.
		resourcesDir = resourceManager.getResourcesDir();

		//attribute style allows to overwrite default 'component.css'
		if (style == null)
			style = widgetDir+"/component.css";
		else
			style = IOUtil.resolvePath(widgetDir,style);
		
		//attribute template allows to overwrite default 'component.html'
		if (template == null) {
			String templatePath = widgetDir+"/component.htm";
			if (isWidgetWebAppResource ? resourceManager.isWebAppResource(templatePath) : resourceManager.isClassPathResource(templatePath))
				template = templatePath;
			else {
				templatePath += "l";
				if (isWidgetWebAppResource ? resourceManager.isWebAppResource(templatePath) : resourceManager.isClassPathResource(templatePath))
					template = templatePath;
				else
					throw new WidgetResourceException("Failed to locate '"+templatePath+"' for widget '"+name+"'");
			}
        } else {
        	template = IOUtil.resolvePath(widgetDir,template);
        }

		widgetConfig = globalConfig.getConfig(name,widgetDir);
		libs = widgetConfig.getLibs();
		styles = widgetConfig.getStyles();
		
		absoluteWidgetPath = isWidgetWebAppResource ?
				uriManager.buildWebAppResourceReference(ajx, widgetDir)
				: uriManager.buildClassPathResourceReference(ajx,widgetDir); 
		globalWebRoot = isGlobalConfigWebAppResource ? uriManager.buildWebAppResourceReference(ajx,"") : uriManager.buildClassPathResourceReference(ajx,"");

                
		//attribute xhpmapping is global defaults to '/xhp' or init-parameter
		if ((xhpMapping=ajx.getXhpMapping())==null)
			xhpMapping = "/xhp";
		AjaxCommonRenderer.getLogger().finest("uuid: '"+uuid
				+"', widgettype: '"+name
				+"', globalWebRoot: '"+globalWebRoot
				+"', resourcesDir: '"+resourcesDir
				+"', widgetdir: '"+widgetDir
				+"', absoluteWidgetPath: '"+absoluteWidgetPath
				//TODO log libs & styles?
				+"', template: '"+template
				+"', style: '"+style
				+"', script: '"+script
				+"', xhpMapping: '"+xhpMapping
				+"', development-mode: '"+(ajx.getUseCache() ? "off'" : "on'"));
	}

	public void doProcess() throws IOException, AjaxException {
		try {
			init();
			// bootstrapscript:
			if (!ajx.isBootstrapScriptWritten()) {
				writeLib(resourcesDir+"/jmaki.js",isGlobalConfigWebAppResource);
				writeScript("jmaki.webRoot='" + ajx.getApplicationRoot() + "';"
						+ "jmaki.resourcesRoot ='" + resourcesDir
						+ "';" + "jmaki.xhp ='" + xhpMapping
						+ "';");
				ajx.setBootstrapScriptWritten(true);
			}
			// scriptlibraries specified in config
			while (libs.hasNext()) {
				String lib = (String)libs.next();
				if (!ajx.isLibraryWritten(lib)) {
					Iterator apiKeys = widgetConfig.getApiKeys();
					while (apiKeys.hasNext()) {
						ApiKey apiKey = (ApiKey)apiKeys.next();
						String targetURL = apiKey.getUrl();
						if ((targetURL != null)
								&& targetURL.equals(globalWebRoot) // TODO potential issue in mixed (ClassPath+WebApp) environment
								|| targetURL.equals("*")) {
							lib += apiKey.getKey();
							break;
						}
					}

					String preload = null;
					if ((preload = widgetConfig.getPreLoad()) != null)
						writeScript(preload);
					writeLib(lib); // location (webapp/classpath) of libs is often not the same as widget-resources
					String postload = null;
					if ((postload = widgetConfig.getPostLoad()) != null)
						writeScript(postload);
					ajx.setLibraryWritten(lib);
				}
			}
			// stylesheets specified in config
			while (styles.hasNext()) {
				String style = (String)styles.next();
				if (!ajx.isStyleWritten(style)) {
					writeStyle(style,isWidgetWebAppResource);
					ajx.setStyleWritten(style);
				}
			}
			//attribute 'style' or default 'component.css'
			if (!ajx.isStyleWritten(style)) {
				writeStyle(style,isWidgetWebAppResource);
				ajx.setStyleWritten(style);
			}
			//attribute 'script' or default 'component.js'
			if (!ajx.isLibraryWritten(script)) {
				writeLib(script,isWidgetWebAppResource);
				ajx.setLibraryWritten(script);
			}

			//attribute 'template' or default 'component.html'
			writeTemplate(template);

			// generate the object literal to configure the widget
			widgetProperties = new HashMap();
			widgetProperties.put("uuid", uuid);
			widgetProperties.put("name", name);
			widgetProperties.put("widgetDir", absoluteWidgetPath);
			if (selected != null) {
				widgetProperties.put("selected", selected);
			}
			if (service != null) {
				widgetProperties.put("service", service);
			}
			if (valueCallback != null) {
				widgetProperties.put("valueCallback", valueCallback);
			}
			if (value != null) {
				widgetProperties.put("value", value);
			}
			if (args != null) {
				widgetProperties.put("args", args);
			}
			writeScript("jmaki.addWidget("
					+ IOUtil.createJavaScriptObject(widgetProperties) + ");");
		} finally {
			if (widgetProperties!=null) {
				widgetProperties.clear();
				widgetProperties=null;
			}
			if (!this.ajx.getUseCache())
				this.ajx.discardConfig();
			this.ajx = null;
			this.widgetConfig = null;
			this.resourceManager = null;
			this.uriManager = null;
			this.out = null;
			this.libs = null;
			this.styles = null;
                        this.template = null;
                        this.templateURL = null;
                        this.name = null;
                        this.script = null;
                        this.widgetDir = null;
		}
	}
	
	private void writeTemplate(String template) throws IOException {

		URL templateURL = isWidgetWebAppResource ? resourceManager.locateWebAppResource(template) : resourceManager.locateClassPathResource(template); 
        StringBuffer templateBuffer = null;
		try {                    
			templateBuffer = IOUtil.loadResource(templateURL.openStream(),null);
		} catch (IOException ioe) {
                    ioe.printStackTrace();
			throw new WidgetResourceException("failed to load 'component.htm' for widget '"+name+"'",ioe);
		}
		IOUtil.replace(templateBuffer, "${uuid}", uuid);
		// default service URL to the name
		if (service != null) {
			IOUtil.replace(templateBuffer, "${service}", service);
		}
//		TODO value from valuebinding: there must be a better way - polling the value via separate request from service-endpoint is undesirable. The value should be sent to widget on initial load if possible,
		if (value != null)
			if (value.startsWith("@{"))
//				replace value bindings in templates with blanks
				IOUtil.replace(templateBuffer, "${value}", "");
			else
				IOUtil.replace(templateBuffer, "${value}", value);
		IOUtil.replace(templateBuffer, "${webRoot}", globalWebRoot); // TODO potential issue in mixed (ClassPath+WebApp) environment
		IOUtil.replace(templateBuffer, "${widgetDir}", absoluteWidgetPath);
		out.write(templateBuffer.toString());
	}
	
	private void writeScript(String script) throws IOException {
		out.write("<script type=\"text/javascript\">" + script + "</script>\n");
	}

	private void writeLib(String lib) throws IOException {
		writeLib(lib,resourceManager.isWebAppResource(lib));
	}
	
	private void writeLib(String lib, boolean isWebAppResource) throws IOException {
		if (!IOUtil.isExternalUri(lib))
			lib = isWebAppResource ? uriManager.buildWebAppResourceReference(ajx,lib) : uriManager.buildClassPathResourceReference(ajx,lib);
		out.write("<script type=\"text/javascript\" src=\""	+ lib + "\"></script>\n");
	}

	private void writeStyle(String style, boolean isWebAppResource) throws IOException {
		if (!IOUtil.isExternalUri(style))
			style = isWebAppResource ? uriManager.buildWebAppResourceReference(ajx, style) : uriManager.buildClassPathResourceReference(ajx, style);
		out.write("<link rel=\"stylesheet\" type=\"text/css\" href=\"" + style + "\"></link>\n");
	}

    private static Logger logger;

    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("jmaki.runtime.Log");
        }
        return logger;
    }
}
