package jmaki.runtime;

import java.util.HashSet;
import java.io.Writer;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.faces.context.FacesContext;
import jmaki.runtime.config.GlobalConfig;
import jmaki.runtime.jsf.Util;


/**
 * @author Norbert von Truchsess
 */
public class AjaxContext {
    
    private Writer out = null;
    private HttpServletRequest request = null;
    private HttpServletResponse response = null;
    private ServletContext ctx = null;
    
	private HashSet writtenLibs = new HashSet();
	private HashSet writtenStyles = new HashSet();
	private boolean bootstrapScriptWritten = false;
	
	private String webRoot = null;
	private String applicationRoot = null;
	private String servletRoot = null;
	private String xhpMapping = null;

	private GlobalConfig config;
	private ResourceManager resourceManager;
	private UriManager uriManager;

	private boolean isJsf = false;
	private boolean isExtensionMapped;
	
	private int counter = 0;
	
	private boolean useCache;
    
	private static final String GLOBALOBJECTS = "jmaki_globalobjects";
	private static final String AJAXCONTEXT = "jmaki_ajaxcontext";

	public static final AjaxContext getInstance(ServletContext ctx, HttpServletRequest req, HttpServletResponse res, Writer out) throws AjaxException {
		AjaxContext ajaxContext = null;
		if ((ajaxContext=(AjaxContext)req.getAttribute(AJAXCONTEXT))==null) {
			ajaxContext = new AjaxContext(ctx,req,res,out);
			req.setAttribute(AJAXCONTEXT,ajaxContext);
		}
		if (!ajaxContext.useCache) ajaxContext.config = new GlobalConfig(ajaxContext.resourceManager);
		return ajaxContext;
	}
	
        private AjaxContext(ServletContext ctx,
                HttpServletRequest req,
                HttpServletResponse res,
                Writer out) throws AjaxException {
            
            AjaxGlobalObjects globalObjects = null;
            if ((globalObjects = (AjaxGlobalObjects)ctx.getAttribute(GLOBALOBJECTS))==null) {
                synchronized(this) { // double check to avoid race-condition
                    if ((globalObjects = (AjaxGlobalObjects)ctx.getAttribute(GLOBALOBJECTS))==null) {
                        globalObjects = new AjaxGlobalObjects();
                		try {
                			if (FacesContext.getCurrentInstance()!=null) {
                                globalObjects.setIsJsf(this.isJsf=true);
                			}
                		} catch (Exception e) {
                		}
                        String useCacheParam = ctx.getInitParameter(AjaxInitParameters.DEVELOPMENTMODE);
                        boolean useCache = useCacheParam == null || !useCacheParam.equalsIgnoreCase("true");
                        globalObjects.setUseCache(useCache);
                        globalObjects.setResourceManager(resourceManager = new ResourceManager(ctx,useCache));
                        if (useCache) globalObjects.setGlobalConfig(new GlobalConfig(resourceManager));
                        globalObjects.setUriManager(getNewUriManagerInstance(ctx));
                        globalObjects.setXhpMapping(ctx.getInitParameter(AjaxInitParameters.XHPMAPPING));
                        ctx.setAttribute(GLOBALOBJECTS,globalObjects);
                    }
                }
            }
            if (this.isJsf=globalObjects.getIsJsf()) {
            	FacesContext context = FacesContext.getCurrentInstance();
            	String facesMapping = Util.getFacesMapping(context);
            	this.isExtensionMapped = facesMapping.startsWith(".");
            }
            this.useCache = globalObjects.getUseCache();
            this.resourceManager = globalObjects.getResourceHandler();
            if (this.useCache) this.config = globalObjects.getGlobalConfig();
            this.uriManager = globalObjects.getUriManager();
            this.xhpMapping = globalObjects.getXhpMapping();
            this.request = req;
            this.response = res;
            this.ctx = ctx;
            this.out = out;
        }
	
	public void discardConfig() throws AjaxException {
		if (useCache) throw new AjaxException("calling discardConfig() is illegal if not in development-mode");
		this.config.discard();
	}
	
	private UriManager getNewUriManagerInstance(ServletContext ctx) throws AjaxException {
		String uriManagerFactoryClassName = ctx.getInitParameter(AjaxInitParameters.URIMANAGERFACTORY);
		if (uriManagerFactoryClassName == null) {
			if (isJsf())
				uriManagerFactoryClassName = UriManagerFactory.DEFAULT_JSF_URIMANAGERFACTORY;
			else
				uriManagerFactoryClassName = UriManagerFactory.DEFAULT_JSP_URIMANAGERFACTORY;
		}
		try {
			Class clasz = Class.forName(uriManagerFactoryClassName);
			return ((UriManagerFactory) clasz.newInstance()).getInstance(ctx); 
		} catch (Exception e) {
			throw new AjaxException("failed to instantiate UriManagerFactory '"+uriManagerFactoryClassName+"'",e);
		}
	}
    
    public Writer getWriter() {return out;}
    public HttpServletRequest getRequest(){return request;}
    public HttpServletResponse getResponse(){return response;}
    public ServletContext getContext() {return ctx;}
    
	public GlobalConfig getConfig() {return config;}
	public ResourceManager getResourceHandler() {return resourceManager;}
	public UriManager getUriManager() {return uriManager;}
	
	public boolean isBootstrapScriptWritten() {return bootstrapScriptWritten;}
	public void setBootstrapScriptWritten(boolean arg) {bootstrapScriptWritten=arg;}

	public boolean isLibraryWritten(String lib) {return writtenLibs.contains(lib);}
	public void setLibraryWritten(String lib) {writtenLibs.add(lib);}
	
	public boolean isStyleWritten(String style) {return writtenStyles.contains(style);}
	public void setStyleWritten(String style) {writtenStyles.add(style);}
	
	public boolean getUseCache() { return this.useCache; }
	
	public boolean isJsf() { return this.isJsf; }
	public boolean getIsExtensionMapped() { return this.isExtensionMapped; }
	
    public int getUniqueId() {
		return counter++;
	}
    
   /** Working directory is the currrent sub-directory used for
    *  searches and is relative to the context root
    *  this is an empty string if working in the context root
    */
    public String getWorkingDir() {
        String basePath = request.getRequestURI();
         String workingDir = "";
        // find the working directory
        // we know the context root is a / so start from postion 1
        int first = basePath.indexOf("/", 1);
        int last = basePath.lastIndexOf("/");
        // find the base path from which to start looking for resources from
        // this is needed if we are working in subdirectories
        if (first != -1 && last != -1) {
            workingDir = basePath.substring(first, last);
        }
        return workingDir;
    }
    
    public String getWebRoot() {
        if (webRoot == null) {
            // gmurray71 : we need to account for https
            StringBuffer webRootBuffer = new StringBuffer(getScheme() + "://").append(request.getServerName());
            if (request.getServerPort() != 80) {
                webRootBuffer.append(":").append(request.getServerPort());
            }
            webRoot = webRootBuffer.toString();
        }
        return webRoot;
    }
    
    public String getApplicationRoot() {
    	if (applicationRoot == null) {
    	    applicationRoot = getWebRoot()+request.getContextPath();      
    	}
    	return applicationRoot; 
    }

    public String getScheme() {
        return ((HttpServletRequest)request).getScheme();
    }
        
    
    public String getServletRoot() {
    	if (servletRoot == null) {
    		servletRoot = getApplicationRoot()+request.getServletPath();
    	}
    	return servletRoot;
    }
    
    public String getXhpMapping() { return xhpMapping; }
}

